jQuery.sap.declare("sas.stress.archive.model.ArchiveLocation");

jQuery.sap.require("sas.stress.archive.ArchiveUtil");

sap.ui.base.Object.extend("sas.stress.archive.model.ArchiveLocation", {

    constructor: function (type, workgroup) {
        var rb = sas.stress.archive.ArchiveUtil.getBundle();
        this.type = type;
        this.workGroup = "";
        if (sas.stress.archive.model.ArchiveLocation.isUserStorage(type)) {
            this.key = type;
        } else if (!workgroup) {   // this should not happen any more with the new public work group, but just in case
            this.key = sas.stress.archive.model.ArchiveLocation.Key.PublicStorage;
        } else {
            this.key = workgroup;
            this.label = workgroup;
            this.workGroup = workgroup;
        }
        switch (this.key) {
            case sas.stress.archive.model.ArchiveLocation.Key.UserStorage:
            // case sas.stress.archive.model.ArchiveLocation.Key.PublicStorage:   //S1384346: don't interpret per Dan
                this.label = rb.getText("archive.location." + this.key + ".label");
                this.tooltip = rb.getText("archive.location." + this.key  + ".tip");
                break;
            default:
                this.tooltip = rb.getText("archive.location." + type + ".tip.fmt", workgroup);
        }
    }
});

sas.stress.archive.model.ArchiveLocation.prototype.getType = function () {
    return this.type;
};

sas.stress.archive.model.ArchiveLocation.prototype.getWorkGroup = function () {
    return this.workGroup;
};

sas.stress.archive.model.ArchiveLocation.Key = {
    UserStorage:        "UserStorage",
    PublicStorage:      "public",     //S1384346: this is getting from workgroups
    WorkgroupStorage:   "WorkgroupStorage"
};

sas.stress.archive.model.ArchiveLocation.getAllLocations = function () {
    var locations = [
        new sas.stress.archive.model.ArchiveLocation(sas.stress.archive.model.ArchiveLocation.Key.UserStorage),
        // new sas.stress.archive.model.ArchiveLocation(sas.stress.archive.model.ArchiveLocation.Key.WorkgroupStorage, "")  //S1384346: "public will be included in the workgroups
    ];

    _.each(sas.stress.wkbench.StressUtil.getRiskWorkGroups(), function (workGroup) {
        locations.push(new sas.stress.archive.model.ArchiveLocation(sas.stress.archive.model.ArchiveLocation.Key.WorkgroupStorage, workGroup));
    });
    return locations;
};

sas.stress.archive.model.ArchiveLocation.isUserStorage = function (locationKey) {
    return  locationKey === sas.stress.archive.model.ArchiveLocation.Key.UserStorage;
};

sas.stress.archive.model.ArchiveLocation.isWorkGroupStorage = function (locationKey) {
    return  locationKey !== sas.stress.archive.model.ArchiveLocation.Key.UserStorage;
};
