jQuery.sap.declare("sas.stress.config.StressAPI");

sas.stress.config.StressAPI = {

    listHierarchyCategories:function(){
        var deferred = $.Deferred();
        $.getJSON("rest/config/hierarchies/categories", function(categories) {
            deferred.resolve(categories.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    addHierarchyCategory : function(hierarchyCategory){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/config/hierarchies/categories",
            data: JSON.stringify(hierarchyCategory),
            dataType: "json"
        })
            .done(function( hierarchyCategory ) {
                deferred.resolve(hierarchyCategory);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    updateHierarchyCategory : function(hierarchyCategory){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json"
            },
            type: "PUT",
            url: "rest/config/hierarchies/categories/" + hierarchyCategory.getCategoryKey(),
            data: JSON.stringify(hierarchyCategory),
            dataType: "json"
        })
            .done(function() {
                deferred.resolve();
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    deleteHierarchyCategory: function(categoryKey){
        var deferred = $.Deferred();
        $.ajax({
            type: "DELETE",
            url: "rest/config/hierarchies/categories/" + categoryKey
        })
            .done(function() {
                deferred.resolve();
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    listDimensionCategories:function(){
        var deferred = $.Deferred();
        $.getJSON("rest/config/dimensions/categories", function(categories) {
            deferred.resolve(categories.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    addDimensionCategory : function(dimensionCategory){
        var deferred = $.Deferred();
        $.ajax({
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: "rest/config/dimensions/categories",
                data: JSON.stringify(dimensionCategory),
                dataType: "json"
            })
            .done(function( dimensionCategory ) {
                deferred.resolve(dimensionCategory);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    updateDimensionCategory : function(dimensionCategory){
        var deferred = $.Deferred();
        $.ajax({
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json"
                },
                type: "PUT",
                url: "rest/config/dimensions/categories/" + dimensionCategory.getCategoryKey(),
                data: JSON.stringify(dimensionCategory),
                dataType: "json"
            })
            .done(function() {
                deferred.resolve();
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    deleteDimensionCategory: function(categoryKey){
        var deferred = $.Deferred();
        $.ajax({
                type: "DELETE",
                url: "rest/config/dimensions/categories/" + categoryKey
            })
            .done(function() {
                deferred.resolve();
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    ////Gets the list of Dimensions
    //getDimensions : function(){
    //    var deferred = $.Deferred();
    //    $.getJSON("rest/config/dimensions", function(dimensions) {
    //        deferred.resolve(dimensions.items);
    //    }).fail(function( jqxhr, textStatus, error ) {
    //        jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
    //        deferred.reject(jqxhr);
    //    });
    //    return deferred;
    //},

    //Gets the list of Dimensions
    getDimensions : function(){
        var deferred = $.Deferred();

        $.ajax({
            type:'GET',
            url: "rest/config/dimensions",
            dataType: "json",
            accepts: { json: "application/vnd.sas.risk.stress.dimension+json;charset=utf-8" }
        }).done(function(data) {
            deferred.resolve(data.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });

        return deferred;
    },

    //Force a refresh of the midtier metadata user cache
    refreshUserCache : function(){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/config/clearUserGroupCache"
        }).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    // Get workgroup/capability info for the given user
    getUserWorkGroupData:function(userName) {

        var deferred = $.Deferred();
        var userParam = (userName) ? "?userId=" + userName : "";

        $.getJSON("rest/config/groups/" + userParam, function(groups) {
            deferred.resolve(groups);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });

        return deferred;

    },

    //Gets Dimensions and details (levels/members)
    getDimensionsWithDetails : function(){
        var deferred = $.Deferred();
        $.ajax({
            type:'GET',
            url: "rest/config/dimensions",
            dataType: "json",
            accepts: { json: "application/vnd.sas.risk.stress.dimensionDetails+json;charset=utf-8" }
        }).done(function(data) {
            deferred.resolve(data.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    //Gets the list of Hierarchies along with its line items.
    getHierarchyDetails : function(){
        var deferred = $.Deferred();

        $.ajax({
            type:'GET',
            url: "rest/config/hierarchies/details",
            dataType: "json",
            accepts: { json: "application/vnd.sas.risk.stress.hierarchies+json;version2" }
        }).done(function(data) {
            deferred.resolve(data.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    ////Gets the list of Hierarchies along with its line items.
    //getHierarchyDetails : function(){
    //    var deferred = $.Deferred();
    //    $.getJSON("rest/config/hierarchies/details", function(hierarchies) {
    //        deferred.resolve(hierarchies.items);
    //    }).fail(function( jqxhr, textStatus, error ) {
    //        jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
    //        deferred.reject(jqxhr);
    //    });
    //    return deferred;
    //},

    getSpecificHierarchyDetails:function(hierarchyKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/config/hierarchies/"+hierarchyKey+"/details", function(hierarchy) {
            deferred.resolve(hierarchy);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    //Gets the list of Hierarchies
    getHierarchies : function(){
        var deferred = $.Deferred();
        $.getJSON("rest/config/hierarchies", function(hierarchies) {
            deferred.resolve(hierarchies.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    //Gets the list of Hierarchy mappings
    getHierarchyMapPairs : function(){
        var deferred = $.Deferred();
        $.getJSON("rest/config/hierarchies/maps", function(hierarchyMapPairs) {
            deferred.resolve(hierarchyMapPairs.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    addHierarchyMapPairs : function(hierarchyMapPairs){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/config/hierarchies/maps",
            data: JSON.stringify(hierarchyMapPairs),
            dataType: "json"
        })
            .done(function( hierarchyMapPairs ) {
                deferred.resolve(hierarchyMapPairs);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    deleteHierarchyMapPairs : function(hierarchyMapKeys){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json"
            },
            type: "DELETE",
            url: "rest/config/hierarchies/maps/",
            data: JSON.stringify(hierarchyMapKeys),
            dataType: "json"
        })
            .done(function() {
                deferred.resolve();
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    getHierarchyMaps:function(hierarchyKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/config/hierarchies/" + hierarchyKey + "/targetMappings", function(result) {
            deferred.resolve(result);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    updateTargetMappings : function(hierarchyKey, hierarchyMaps){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "PUT",
            url: "rest/config/hierarchies/" + hierarchyKey + "/targetMappings",
            data: JSON.stringify(hierarchyMaps),
            dataType: "json"
        })
            .done(function() {
                deferred.resolve();
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    listHierarchyLoadingTableNames:function(){
        var deferred = $.Deferred();
        $.getJSON("rest/config/sourceTableNames", function(result) {
            deferred.resolve(result.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    listColumnMetas:function(tableName){
        var deferred = $.Deferred();
        $.getJSON("rest/config/listColumnMetas/" + tableName, function(result) {
            deferred.resolve(result.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    listHierarchyCCVars:function(tableName){
        var deferred = $.Deferred();
        $.getJSON("rest/config/ccvars/" + tableName, function(result) {
            deferred.resolve(result.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    addHierarchy : function(hierarchy){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/config/hierarchies",
            data: JSON.stringify(hierarchy),
            dataType: "json"
        })
            .done(function( hierarchy ) {
                deferred.resolve(hierarchy);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    updateHierarchy : function(hierarchy){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "PUT",
            url: "rest/config/hierarchies/" + hierarchy.getHierarchyKey(),
            data: JSON.stringify(hierarchy),
            dataType: "json"
        })
            .done(function(updatedHierarchy) {
                deferred.resolve(updatedHierarchy);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    deleteHierarchy: function(hierarchyKey){
        var deferred = $.Deferred();
        $.ajax({
            type: "DELETE",
            url: "rest/config/hierarchies/" + hierarchyKey
        })
            .done(function() {
                deferred.resolve();
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    addDimension : function(dimension){
        var deferred = $.Deferred();
        $.ajax({
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: "rest/config/dimensions",
                data: JSON.stringify(dimension),
                dataType: "json"
            })
            .done(function( dimension ) {
                deferred.resolve(dimension);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    updateDimension : function(dimension){
        var deferred = $.Deferred();
        $.ajax({
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "PUT",
                url: "rest/config/dimensions/" + dimension.getId(),
                data: JSON.stringify(dimension),
                dataType: "json"
            })
            .done(function(updatedDimension) {
                deferred.resolve(updatedDimension);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    deleteDimension: function(dimensionKey){
        var deferred = $.Deferred();
        $.ajax({
                type: "DELETE",
                url: "rest/config/dimensions/" + dimensionKey
            })
            .done(function() {
                deferred.resolve();
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },

    getHierarchyAccessControls: function (hierarchyKey, liKey) {
        var deferred = $.Deferred();
        var restAPI = "rest/config/hierarchies/" + hierarchyKey;
        if (liKey && liKey.length > 0) {
            restAPI += "/items/" + liKey;
        }
        $.getJSON(restAPI + "/accessControls", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, errorThrown) {
            jQuery.sap.log.error("loading access controls FAILURE: " + errorThrown);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteProcessDefinitions: function(processDefnKey){
        var deferred = $.Deferred();
        $.ajax({
            type: "DELETE",
            url: "rest/tasks/definitions/" + processDefnKey
        })
            .done(function( result) {
                deferred.resolve(result);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },
    
    uploadProcessDefinitions : function(attachment) {
    	var formData=new FormData();
		formData.append('file',attachment);
        formData.append('name', attachment.name);

        var deferred = $.Deferred();
        $.ajax({
        	type: "POST",
            url: "rest/tasks/definitions",
            data: formData,
            processData: false,
	    	contentType: false,
	    	dataType:"text",
            mimeType: "multipart/form-data",
	    	cache: false
        })
            .done(function( result ) {
                deferred.resolve(result);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },
    
    listSheetConfigs:function(projectKey){
        var deferred = $.Deferred();
        $.getJSON("rest/config/projects/" + projectKey + "/sheets", function(result) {
            deferred.resolve(result.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject();
        });
        return deferred;
    },

    addSheet:function(sheetConfig){
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/config/projects/" + sheetConfig.getProjectKey() + "/sheets",
            data: JSON.stringify(sheetConfig),
            dataType: "json"
        })
            .done(function( result ) {
                deferred.resolve(result);
            })
            .fail(function( jqxhr, textStatus, error ) {
                jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        return deferred;
    },
    loadSheetTemplates : function(excludePackage) {
        var deferred = $.Deferred();
        if (!excludePackage || excludePackage === null) {
            excludePackage = false;
        }
        $.getJSON("rest/config/sheets/templates?excludePackage=" + excludePackage, function(templates) {
            deferred.resolve(templates.items);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    createSheetTemplate : function(sheetTemplate) {
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/config/sheets/templates",
            data: JSON.stringify(sheetTemplate),
            dataType: "json"
        }).done(function(result){
            deferred.resolve(result);
        }).fail(function(jqxhr, textStatus, error){
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;  	
    },
    updateSheetTemplate : function(sheetTemplate) {
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "PUT",
            url: "rest/config/sheets/templates/" + sheetTemplate.id,
            data: JSON.stringify(sheetTemplate),
            dataType: "json"
        }).done(function(updated) {
            deferred.resolve(updated);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    getSheetTemplateAccessControls: function (templateId) {
        var deferred = $.Deferred();
        $.getJSON("rest/config/sheets/templates/" + templateId + "/accessControls", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, errorThrown) {
            jQuery.sap.log.error("loading access controls FAILURE: " + errorThrown);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    saveSheetTemplateAccessControls: function (templateId, workgroups) {
        var deferred = $.Deferred();
        var accessControlsJson = { workgroups: workgroups };
        $.ajax({
                   headers: {
                       Accept: "application/json",
                       "Content-Type": "application/json;charset=utf-8"
                   },
                   type: "PUT",
                   url: "rest/config/sheets/templates/" + templateId + "/accessControls",
                   data: JSON.stringify(accessControlsJson),
                   dataType: "json"
               }
        ).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteSheetTemplates : function(sheetTemplateKeys) {
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json"
            },
            type: "DELETE",
            url: "rest/config/sheets/templates",
            data: JSON.stringify(sheetTemplateKeys),
            dataType: "json"
        }).done(function() {
            deferred.resolve();
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;  	
    },
    loadSheetCategories : function() {
	    var deferred = $.Deferred();
	    $.getJSON("rest/config/sheets/categories", function(cats) {
		    deferred.resolve(cats.items);
	    }).fail(function( jqxhr, textStatus, error ) {
			jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject();
        });
	    return deferred;
    },
    createSheetCategory : function(sheetCategory) {
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "POST",
            url: "rest/config/sheets/categories",
            data: JSON.stringify(sheetCategory),
            dataType: "json"
        }).done(function(result){
            deferred.resolve(result);
        }).fail(function(jqxhr, textStatus, error){
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;  	
    },
    updateSheetCategory : function(sheetCategoryKey, sheetCategory) {
        var deferred = $.Deferred();
        $.ajax({
            headers: {
                Accept: "application/json",
                "Content-Type": "application/json;charset=utf-8"
            },
            type: "PUT",
            url: "rest/config/sheets/categories/" + sheetCategoryKey,
            data: JSON.stringify(sheetCategory),
            dataType: "json"
        }).done(function(updated) {
            deferred.resolve(updated);
        }).fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    deleteSheetCategory : function(sheetCategoryKey) {
        var deferred = $.Deferred();
        $.ajax({
            type: "DELETE",
            url: "rest/config/sheets/categories/" + sheetCategoryKey
        })
        .done(function() {
            deferred.resolve();
        })
        .fail(function( jqxhr, textStatus, error ) {
            jQuery.sap.log.error( "Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;  	
    }
};