jQuery.sap.require("sap.ui.core.CustomData");
jQuery.sap.require("sap.ui.core.Orientation");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.List");
jQuery.sap.require("sas.hc.m.StandardListItem");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.ui.layout.Splitter");
jQuery.sap.require("sas.hc.ui.layout.SplitterLayoutData");
jQuery.sap.require("sas.hc.ui.table.TreeTable");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.widget.CategoryFilter");

sap.ui.jsview("sas.stress.config.view.ProjectConfiguration", {
    getControllerName: function () {
        return "sas.stress.config.view.ProjectConfiguration";
    },

    createContent: function (controller) {
        this.setHeight("100%");

        var categoriesPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
            topPadding: 0,
            leftPadding: "0.5rem",
            content: this._createCategoryFilter(controller)
        }).addStyleClass("riskFacetPanel");

        var hierarchyPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
            rightPadding: "0.5rem",
            content: this._createHierarchiesTreeTable(controller)
        });

        categoriesPanel.setLayoutData(new sas.hc.ui.layout.SplitterLayoutData({
            size: sas.stress.wkbench.widget.CategoryFilter.DEFAULT_WIDTH,
            minSize: 150
        }));

        return new sas.hc.ui.layout.Splitter({
            contentAreas: [
                hierarchyPanel,
                categoriesPanel
            ],
            orientation: sap.ui.core.Orientation.Horizontal,
            width: "100%",
            height: "100%"
        });

    },

    _createHierarchyTableToolbar: function (controller) {

        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");

        this.projectAccessControlsMenu = new sas.hc.m.MenuItem({
            id: this.createId("hierarchiesTreeTable.accessControlsButton"),
            press: $.proxy(controller.hierarchyAccessControlsButtonPress, controller),
            enabled: false,
            text: rb.getText("hierarchiesTreeTable.workGroupButton.tip")
        });

        this.refreshMenu = new sas.hc.m.MenuItem({
            id: this.createId("hierarchiesTreeTable.refreshButton"),
            enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.READ),
            press: $.proxy(controller.refreshList, controller),
            text: rb.getText("button.refresh.action.label")
        });

        var overflowMenu = new sas.hc.m.Menu({
            items: [
                this.projectAccessControlsMenu,
                this.refreshMenu
            ]
        });

        var overflowButton = new sas.hc.m.MenuButton({
            icon: sas.icons.HC.OVERFLOW,
            menu: overflowMenu
        });

        this.searchField = new sas.hc.m.SearchField({
            liveChange:$.proxy(controller.hierarchySearch, controller),
            search:$.proxy(controller.hierarchySearch, controller),
            filterMode: true
        });
        return new sas.hc.m.ResponsiveToolbar({
            content: [
                this.searchField,
                new sas.hc.m.ToolbarSpacer(),
                new sas.hc.m.Button({
                    id: this.createId("hierarchiesTreeTable.expandButton"),
                    icon: sas.icons.HC.SHOWALL,
                    tooltip: rb.getText("hierarchiesTreeTable.expandButton.tip"),
                    enabled: false, //Initially false, will be enabled when a hierarchy is selected.
                    press: $.proxy(controller.hierarchiesTreeTableExpandButtonPress, controller)
                }),
                new sas.hc.m.Button({
                    id: this.createId("hierarchiesTreeTable.collapseButton"),
                    icon: sas.icons.HC.HIDEALL,
                    tooltip: rb.getText("hierarchiesTreeTable.collapseButton.tip"),
                    enabled: false, //Initially false, will be enabled when a hierarchy is selected.
                    press: $.proxy(controller.hierarchiesTreeTableCollapseButtonPress, controller)
                }),
                new sas.hc.m.ToolbarSeparator(),
                new sas.hc.m.Button({
                    id: this.createId("hierarchiesTreeTable.createButton"),
                    icon: sas.icons.HC.NEWSTARBURST,
                    tooltip: rb.getText("hierarchiesTreeTable.createButton.tip"),
                    press: $.proxy(controller.hierarchiesTreeTableCreateButtonPress, controller),
                    enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.CREATE)
                }),
                new sas.hc.m.Button({
                    id: this.createId("hierarchiesTreeTable.editButton"),
                    icon: sas.icons.HC.EDIT,
                    tooltip: rb.getText("hierarchiesTreeTable.editButton.tip"),
                    enabled: false, //Initially false, will be enabled when a hierarchy is selected.
                    press: $.proxy(controller.hierarchiesTreeTableEditButtonPress, controller)
                }),
                new sas.hc.m.Button({
                    id: this.createId("hierarchiesTreeTable.mapButton"),
                    icon: sas.icons.HC.MAPPEDVALUES,
                    tooltip: rb.getText("hierarchiesTreeTable.configureMappingButton.tip"),
                    enabled: false, //Initially false, will be enabled when a hierarchy is selected.
                    press: $.proxy(controller.hierarchiesTreeTableMapButtonPress, controller)
                }),
                new sas.hc.m.Button({
                    id: this.createId("hierarchiesTreeTable.deleteButton"),
                    icon: sas.icons.HC.DELETE,
                    tooltip: rb.getText("hierarchiesTreeTable.deleteButton.tip"),
                    enabled: false, //Initially false, will be enabled when a hierarchy is selected.
                    press: $.proxy(controller.hierarchiesTreeTableDeleteButtonPress, controller)
                }),
                overflowButton
            ]
        });

    },

    _createHierarchiesTreeTable: function (controller) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        this.hierarchiesTreeTable = new sas.hc.ui.table.TreeTable({
            columns: [
                new sas.hc.ui.table.Column({
                    id: this.createId("name"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.name.header.title") }),
                    template: new sas.hc.m.Text({ text: "{name}", tooltip: "{name}", wrapping: false }),
                    sortProperty: "name",
                    mandatory: true,
                    width: "15rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("version"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.version.header.title") }),
                    template: new sas.hc.m.Text({ text: "{version}", tooltip: "{version}", wrapping: false }),
                    sortProperty: "version"
                    //width: "10rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("workgroups"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.workGroups.header.title") }),
                    template: new sas.hc.m.Text({ text: "{workGroupString}", tooltip: "{workGroupString}", wrapping: false })
                    //width: "10rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("dimensionName"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.dimensionName.header.title") }),
                    template: new sas.hc.m.Text({ text: "{dimensionName}", tooltip: "{dimensionName}", wrapping: false }),
                    sortProperty: "dimensionName"
                    //width: "10rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("defaultHierarchy"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.defaultHierarchy.header.title") }),
                    template: new sas.hc.ui.core.Icon({
                        src: {
                            path: "defaultHierarchy",
                            formatter: function (bValue) {
                                return bValue ? sas.icons.HC.COMMITCHANGES : sas.icons.HC.BLANK;
                            }
                        },
                        tooltip: {
                            path: "defaultHierarchy",
                            formatter: function (bValue) {
                                return bValue ? rb.getText("hierarchiesTreeTable.column.aggregation.true.tip") : " ";
                            }
                        }
                    }),
                    hAlign: sap.ui.core.HorizontalAlign.Center,
                    width: "4rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("liKey"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.id.header.title") }),
                    template: new sas.hc.m.Text({ text: "{liKey}", tooltip: "{liKey}", wrapping: false }),
                    width: "10rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("level"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.level.header.title") }),
                    template: new sas.hc.m.Text({ text: "{level}", tooltip: "{level}", wrapping: false })
                    //width: "10%"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("contra"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.contra.header.title") }),
                    template: new sas.hc.ui.core.Icon({
                        src: {
                            path: "contra",
                            formatter: function (bValue) {
                                return bValue ? sas.icons.HC.COMMITCHANGES : sas.icons.HC.BLANK;
                            }
                        },
                        tooltip: {
                            path: "contra",
                            formatter: function (bValue) {
                                return bValue ? rb.getText("hierarchiesTreeTable.column.contra.true.tip") : " ";
                            }
                        }
                    }),
                    hAlign: sap.ui.core.HorizontalAlign.Center,
                    width: "4rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("aggregationType"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.aggregation.header.title") }),
                    template: new sas.hc.m.Text({ text: "{aggregationType}", tooltip: "{aggregationType}", wrapping: false }),
                    sortProperty: "aggregationType",
                    width: "4rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("format"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.format.header.title") }).addStyleClass("rightAlignedHeaderLabel"),
                    template: new sas.hc.m.Text({ text: "{format}", tooltip: "{format}", wrapping: false }),
                    hAlign: sap.ui.core.HorizontalAlign.Right,
                    width: "4rem"
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("formula"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.column.formula.header.title") }),
                    template: new sas.hc.ui.core.Icon({
                        src: {
                            path: "formulaCode",
                            formatter: function (bValue) {
                                return bValue ? sas.icons.HC.COMMITCHANGES : sas.icons.HC.BLANK;
                            }
                        },
                        tooltip: {
                            path: "formulaCode",
                            formatter: function (bValue) {
                                return bValue ? rb.getText("hierarchiesTreeTable.column.formula.true.tip") : " ";
                            }
                        }
                    }),
                    hAlign: sap.ui.core.HorizontalAlign.Center,
                    width: "4rem",
                    visible: false
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("dateModified"),
                    label: rb.getText("hierarchiesTreeTable.dateModified.label.txt"),
                    template: new sas.hc.m.Text({
                        text: {
                            path: "updatedTimestamp",
                            type: new sap.ui.model.type.Date({source: {pattern: "timestamp"}, style: "medium"})
                        },
                        tooltip: {
                            path: "updatedTimestamp",
                            type: new sap.ui.model.type.DateTime({source: {pattern: "timestamp"}})
                        },
                        maxLines: 2
                    }),
                    //width: "10%",
                    visible: false
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("updateBy"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.modifiedBy.label.txt")}),
                    template: new sas.hc.m.Text({ text: "{updatedByUser}", tooltip: "{updatedByUser}", wrapping: false }),
                    //width: "10%",
                    visible: false
                }),
                new sas.hc.ui.table.Column({
                    id: this.createId("description"),
                    label: new sas.hc.m.Label({ text: rb.getText("hierarchiesTreeTable.description.label.txt")}),
                    template: new sas.hc.m.Text({ text: "{description}", tooltip: "{description}", wrapping: false }),
                    //width: "15%",
                    visible: false
                })
            ],
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            rowSelectionChange: $.proxy(controller.hierarchiesTreeTableRowSelectionChange, controller),
            noData: rb.getText("hierarchiesTreeTable.noData.loading.txt"),
            toolbar: this._createHierarchyTableToolbar(controller)
        });

        return this.hierarchiesTreeTable;
    },

    _createCategoryFilter: function (controller) {
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();

        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        this.categoryFilter = new sas.stress.wkbench.widget.CategoryFilter({
            actionButtons: [
                new sas.hc.m.Button({
                    icon: sas.icons.HC.NEWSTARBURST,
                    tooltip: rb.getText("categoriesList.createButton.tip"),
                    press: $.proxy(controller.categoriesListCreateButtonPress, controller),
                    enabled:capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.HIERARCHY.CREATE)
                }),
                new sas.hc.m.Button({
                    id: this.createId("categoriesList.editButton"),
                    icon: sas.icons.HC.EDIT,
                    tooltip: rb.getText("categoriesList.editButton.tip"),
                    press: $.proxy(controller.categoriesListEditButtonPress, controller),
                    enabled:false
                }),
                new sas.hc.m.Button({
                    id: this.createId("categoriesList.deleteButton"),
                    icon: sas.icons.HC.DELETE,
                    tooltip: rb.getText("categoriesList.deleteButton.tip"),
                    press: $.proxy(controller.categoriesListDeleteButtonPress, controller),
                    enabled:false
                })
            ],
            itemsPath: "/categories",
            itemTemplate: new sas.hc.m.StandardListItem({
                title: "{name}",
                counter: "{hierarchiesCount}"
            }),
            selectionChange: $.proxy(controller.categoriesListSelectionChange, controller)
        });

        return this.categoryFilter;
    },

    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    }
});