jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.data.model.DataLoadHistoryItem");
jQuery.sap.require("sas.stress.wkbench.LeftNavManager");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");

sap.ui.controller("sas.stress.data.view.ProjectDataEditor", {

    onInit: function () {
        this.router = sap.ui.core.routing.Router.getRouter("router");
        this.router.attachExactRouteMatched("ProjectData", this._init, this);
        this.navChecker = sas.stress.wkbench.StressUtil.createNavChecker(true, this.getView());
        new sas.risk.common.utils.LayoutService(this.getView().projectDataLoadHistoryTable, "ProjectDataLoadHistoryTable");

        // show groove bar
        sas.stress.wkbench.LeftNavManager.getInstance().showGrooveBar(sas.stress.wkbench.LeftNavManager.PROJECTS);
    },
    _init: function (event) {
        var controller = this;
        var view = this.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({bundleName: "sas.stress.data.messagebundle"});
        view.setModel(i18N, "i18N");
        var projectId = event.getParameter("arguments").projectId;

        // S1273684: don't use the cached project since it might be stale or deleted by others
        //controller.project = sas.stress.wkbench.model.Project.getProject(projectId);
        //if (!controller.project) {	// no it is not in the cache
            sas.stress.wkbench.model.Project.loadProject(projectId, true, false).done(function (project) {
                controller.project = sas.stress.wkbench.model.Project.getProject(projectId);
                controller.afterProjectIsLoaded();
            }).fail(function (jqxhr) {
                sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                controller.getView().setBusy(false);
                sas.stress.wkbench.ViewManager.loadProjectManager();
                //controller.getView().destroy();
            });
        //} else {
        //    controller.afterProjectIsLoaded();
        //}
    },
    afterProjectIsLoaded: function () {
        //Load data in the table
        this._getDataLoadHistory();
    },
    onExit: function () {
        this.router.detachExactRouteMatched("ProjectData", this._init, this);
        // clear the View on the router otherwise the next router.navTo() would fail with a blank page
        this.router.setView("sas.stress.data.view.ProjectDataEditor", null);
        this.navChecker.stop();
    },

    _getDataLoadHistory: function () {
        var view = this.getView();
        var controller = this;
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");
        view.projectDataLoadHistoryTable.setNoData(rb.getText("projectDataLoadHistoryTable.noData.loading.txt"));
        view.projectDataLoadHistoryTable.setBusyIndicatorDelay(0);
        view.projectDataLoadHistoryTable.setBusy(true);
        sas.stress.data.model.DataLoadHistoryItem.getProjectDataLoadHistory(this.project.getId()).done(function (dataLoadHistoryItems) {
            var json = [];
            _.each(dataLoadHistoryItems, function (dataLoadHistoryItem) {
                var translatedSource=dataLoadHistoryItem.getDataPointSource();
                switch (dataLoadHistoryItem.getDataPointSource()) {
                    case "calculatedField":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.calculatedfield.column.value.txt");
                        break;
                    case "sheetOverride":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.sheetoverride.column.value.txt");
                        break;
                    case "dataLoad":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.dataload.column.value.txt");
                        break;
                    case "modelExecution":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.modelExecution.column.value.txt");
                        break;
                    case "rollup":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.rollup.column.value.txt");
                        break;
                    case "excelImport":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.excelimport.column.value.txt");
                        break;
                    case "archiveImport":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.archiveImport.column.value.txt");
                        break;
                    case "allocationOverride":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.allocationOverride.column.value.txt");
                        break;
                    case "externalOverride":
                        translatedSource = rb.getText("projectDataLoadHistoryTable.source.externalOverride.column.value.txt");
                        break;
                }
                json.push({
                    projectIteration: dataLoadHistoryItem.getProjectIteration(),
                    userName: dataLoadHistoryItem.getUserName(),
                    uploadDate: dataLoadHistoryItem.getUploadDate(),
                    fileName: dataLoadHistoryItem.getFileName(),
                    fileLocationType: dataLoadHistoryItem.getFileLocationType(),
                    numRecords: dataLoadHistoryItem.getNumRecords(),
                    fileSize: dataLoadHistoryItem.getFileSize(),
                    dataFormat: dataLoadHistoryItem.getDataFormat(),
                    dataLoadMapName: dataLoadHistoryItem.getDataLoadMapName(),
                    dataPointSource: translatedSource
                });
            });
            // To maintain any sort already applied on the table, if model exists, set data on the model.
            // view.setModel recreates the binding to the model by the Table and the previously applied sort information is lost.
            var model = view.projectDataLoadHistoryTable.getModel("projectDataLoadHistory");
            if (model instanceof sap.ui.model.json.JSONModel) {
                model.setData(json);
            } else {
                var newModel = new sas.risk.common.models.RiskJSONModel(json);
                view.projectDataLoadHistoryTable.setModel(newModel, "projectDataLoadHistory");
                //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
                //var sorters = [new sap.ui.model.Sorter("uploadDate", true),
                //    new sap.ui.model.Sorter("fileName", false)];
                //view.projectDataLoadHistoryTable.getBinding("rows").sort(sorters);
            }
            var viewjson = {
                enabled: !controller.project.isLocked(),
            };
            var viewModel= new sap.ui.model.json.JSONModel(viewjson);
            viewModel.setProperty("/enabled",!controller.project.isLocked());
            if (controller.project) {
                viewModel.setProperty("/projectName", controller.project.getName());
                viewModel.setProperty("/currentIterationNo", controller.project.getCurrentIterationNo());
                viewModel.setProperty("/asOfDate", controller.project.getAsOfDate());
                viewModel.setProperty("/projectIsLocked", controller.project.isLocked());
            }
            view.setModel(viewModel);
            controller.handleProcessOrchestration();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            sas.stress.wkbench.ViewManager.loadProjectManager();
            //controller.getView().destroy();
        }).always(function() {
            view.projectDataLoadHistoryTable.setBusy(false);
            view.projectDataLoadHistoryTable.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                text: "{i18N>dataLoadHistoryTable.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>dataLoadHistoryTable.noData.button.txt}",
                        tooltip: "{i18N>dataLoadHistoryTable.loadDataButton.tip}",
                        press: $.proxy(controller.loadDataButtonPress, controller),
                        enabled: !controller.project.isLocked()
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        });

        var viewModel = new sas.risk.common.models.RiskJSONModel({
            "projectName": this.project.getName(),
            "currentIterationNo": this.project.getCurrentIterationNo(),
            "asOfDate": this.project.getAsOfDate()
        });
        view.setModel(viewModel);

    },

    handleProcessOrchestration: function () {
        var view = this.getView();
        // handle process orchestration commands, if they exist
        if (sap.ui.getCore().getModel("ProjectDataParams")) {
            if (view.loadDataBtn.getEnabled()) {
                view.loadDataBtn.firePress();
            } else {
                // Shouldn't happen currently, but let's be safe.
                sap.ui.getCore().setModel(undefined, "ProjectDataParams");
                console.log("Stress error: Can't handle process orchestration command, add button isn't enabled!");
            }
        }
    },

    loadDataButtonPress: function (event) {
        var controller = this;
        var view = sas.stress.wkbench.ViewManager.loadLoadProjectDataEditor({projectId: controller.project.getId()});
        var i18N = new sap.ui.model.resource.ResourceModel({bundleName: "sas.stress.data.messagebundle"});
        var dialog = new sas.hc.m.Dialog({
            title: i18N.getProperty("loadProjectDataEditor.window.title"),
            contentWidth: sas.stress.wkbench.StressUtil.dialogMedium,
            content: view,
            beginButton: view.okButton,
            endButton: view.cancelButton,
            afterClose: function () {
                dialog.destroy();
            },
            initialFocus: view.cancelButton
        })/*.allowTextSelection(false).addStyleClass("stressPreventTextSelection")*/;
        dialog.setModel(i18N, "i18N");
        view.attachClosing(function () {
            this.destroy();
        }, dialog);
        view.attachDataLoaded(function () {
            //Reload data in the table
            controller._getDataLoadHistory();
        }, dialog);
        dialog.open();
    },

    onFilterChanged: function (e) {
        var value = e.getSource().getValue() || "";
        var projectIterationFilter = new sap.ui.model.Filter("projectIteration", sap.ui.model.FilterOperator.EQ, value);
        var filenameFilter = new sap.ui.model.Filter("fileName", sap.ui.model.FilterOperator.Contains, value);
        var uploadedByFilter = new sap.ui.model.Filter("userName", sap.ui.model.FilterOperator.Contains, value);
        var fileLocationTypeFilter = new sap.ui.model.Filter("fileLocationType", sap.ui.model.FilterOperator.Contains, value);
        var allFilters = new sap.ui.model.Filter([projectIterationFilter, filenameFilter, uploadedByFilter, fileLocationTypeFilter]);
        var binding = this.getView().projectDataLoadHistoryTable.getBinding("rows");
        if (binding !== undefined) {
            binding.filter(allFilters);
        }
    }

});