jQuery.sap.require("sap.ui.core.format.FileSizeFormat");
jQuery.sap.require("sap.ui.core.format.DateFormat");
jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.risk.common.widgets.RiskToolbar");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.risk.common.widgets.RiskPage");
jQuery.sap.require("sas.stress.notification.NotificationUtil");
jQuery.sap.require("sas.stress.wkbench.widget.CustomBreadcrumb");
jQuery.sap.require("sas.stress.wkbench.widget.StressIconBar");

sap.ui.jsview("sas.stress.data.view.ProjectDataEditor", {
    getControllerName: function () {
        return "sas.stress.data.view.ProjectDataEditor";
    },

    createContent: function (controller) {

        this.setHeight("100%");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.data");

        this.projectDataLoadHistoryTable = this._createProjectDataLoadHistoryTable(controller);

        this.breadCrumb = new sas.stress.wkbench.widget.CustomBreadcrumb({
            customData: new sap.ui.core.CustomData({key: "size", value: "3rem"})
        }).addStyleClass("innerBreadCrumb");

        this.breadCrumb.addBreadcrumbItem("Top", rb.getText("projectDataEditor.breadcrumb.label"));

        this.viewLayout = new sas.risk.common.widgets.RiskPage({
            mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel({
                type: sas.risk.common.widgets.WidgetFactory.OBJECT,
                top: new sas.risk.common.widgets.RiskToolbar({
                    homeFunction: function () {
                        sas.stress.wkbench.ViewManager.loadProjectManager();
                    },
                    headerContent: [
                        new sas.hc.m.Label({text: sas.stress.wkbench.StressUtil.getBundle().getText("iteration.label.txt")}).addStyleClass("toolbarLabel"),
                        new sas.hc.m.Label({text: "{/currentIterationNo}"}).addStyleClass("toolbarValue"),
                        new sas.hc.m.Label({text: sas.stress.wkbench.StressUtil.getBundle().getText("workbench.toolbar.asOfDate.label")}).addStyleClass("toolbarLabel"),
                        new sas.hc.m.Label({
                            text: {
                                path: "/asOfDate",
                                formatter: function (asOfDate) {
                                    var dateFormat = new sap.ui.model.type.Date({source: {pattern: "yyyy-MM-dd"}}, {target: {pattern: "MMM dd, yyyy"}}, asOfDate);
                                    return dateFormat.formatValue(asOfDate, "string");
                                }
                            }
                        }).addStyleClass("toolbarValue"),
                        sas.stress.notification.NotificationUtil.createNotificationButton()
                    ],
                    title: new sas.hc.m.Label({text: "{/projectName}"})
                }),
                content: new sas.risk.common.widgets.RiskPage({
                    mainContent: sas.risk.common.widgets.WidgetFactory.buildPanel({
                        top: this.breadCrumb,
                        content: this.projectDataLoadHistoryTable
                    }),
                    iconBar: new sas.stress.wkbench.widget.StressIconBar({
                        section: sas.stress.wkbench.widget.StressIconBar.DATA
                    })
                })
            })
        });

        return this.viewLayout;

    },

    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    },

    _createTableHeader: function (controller) {
        this.loadDataBtn = new sas.hc.m.Button({
            id: "ProjectDataEditor.loadDataButton",
            icon: sas.icons.HC.IMPORT,
            tooltip: "{i18N>projectDataLoadHistoryTable.loadDataButton.tip}",
            press: $.proxy(controller.loadDataButtonPress, controller),
            enabled:"{/enabled}"
        });
        return new sas.hc.m.ResponsiveToolbar({
            content: [
                this._createSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                this.loadDataBtn
            ]
        });
    },

    _createSearchField: function (controller) {
        return searchField = new sas.hc.m.SearchField({
            id: "projectDataLoadHistoryTableSearchFld",
            liveChange: $.proxy(controller.onFilterChanged, controller),
            search: $.proxy(controller.onFilterChanged, controller),
            filterMode: true
        });
    },

    _createProjectDataLoadHistoryTable: function (controller) {
        var table = new sas.hc.ui.table.Table({
            id: "projectDataLoadHistoryTable",
            editable: false,
            width: "100%",
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
            showTableOptions: false,
            //noData: "{i18N>dataLoadHistoryTable.noData.loading.txt}",   // setNoData() in controller
            toolbar: this._createTableHeader(controller)
        });

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-filename"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.filename.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>fileName}",
                text: "{projectDataLoadHistory>fileName}"
            }),
            sortProperty: "fileName",
            width: "15rem"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-projectIteration"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.projectIteration.header.title}"}).addStyleClass("rightAlignedHeaderLabel"),
            template: new sas.hc.m.Text({
                textAlign: "Right",
                text: "{projectDataLoadHistory>projectIteration}"
            }),
            sortProperty: "projectIteration",
            hAlign: sap.ui.core.HorizontalAlign.Right,
            width: "6rem"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-uploadedBy"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.uploadedBy.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>userName}",
                text: "{projectDataLoadHistory>userName}"
            }),
            sortProperty: "userName"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-loadDate"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.loadDate.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "projectDataLoadHistory>uploadDate",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                },
                text: {
                    path: "projectDataLoadHistory>uploadDate",
                    formatter: function (dttm) {
                        if (!dttm) return;
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                }
            }),
            sortProperty: "uploadDate"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-fileLocationType"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.fileLocationType.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>fileLocationType}",
                text: "{projectDataLoadHistory>fileLocationType}"
            }),
            sortProperty: "fileLocationType"
        }));
        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-fileSize"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.fileSize.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "projectDataLoadHistory>fileSize",
                    formatter: function (fileSizeInBytes) {
                        if (fileSizeInBytes < 0) {
                            return "";
                        }
                        var fileSizeFormatter = sap.ui.core.format.FileSizeFormat.getInstance({
                            binaryFilesize: false,
                            maxFractionDigits: 2
                        });
                        return fileSizeFormatter.format(fileSizeInBytes);
                    }
                },
                text: {
                    path: "projectDataLoadHistory>fileSize",
                    formatter: function (fileSizeInBytes) {
                        if (fileSizeInBytes < 0) {
                            return "";
                        }
                        var fileSizeFormatter = sap.ui.core.format.FileSizeFormat.getInstance({
                            binaryFilesize: false,
                            maxFractionDigits: 2
                        });
                        return fileSizeFormatter.format(fileSizeInBytes);
                    }
                }
            }),
            sortProperty: "fileSize"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-dataFormat"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.dataFormat.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>dataFormat}",
                text: "{projectDataLoadHistory>dataFormat}"
            }),
            sortProperty: "dataFormat"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-dataLoadMapName"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.dataLoadMapName.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>dataLoadMapName}",
                text: "{projectDataLoadHistory>dataLoadMapName}"
            }),
            sortProperty: "dataLoadMapName"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id:this.createId("projectDataLoadHistoryTable-dataPointSource"),
            label: new sas.hc.m.Label({text: "{i18N>projectDataLoadHistoryTable.column.dataPointSource.header.title}"}),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: "{projectDataLoadHistory>dataPointSource}",
                text: "{projectDataLoadHistory>dataPointSource}"
            }),
            sortProperty: "dataPointSource"
        }));

        table.bindRows("projectDataLoadHistory>/");

        //S1256573: Default sort by Name/Version asc.
        var bindingInfo = table.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [
                new sap.ui.model.Sorter("uploadDate", true),
                new sap.ui.model.Sorter("fileName", false)];
            bindingInfo.sorter = sorters;
        }
        return table;
    }
});
