jQuery.sap.declare('sas.stress.wkbench.CellEvaluationUtil');
jQuery.sap.require("sas.stress.wkbench.parser.Parser");
jQuery.sap.require("sas.stress.wkbench.parser.Lexer");
jQuery.sap.require("sas.stress.wkbench.parser.ExpressionUtil");
jQuery.sap.require("sas.stress.wkbench.model.CellRef");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.parser.Math");
jQuery.sap.require("sas.stress.wkbench.parser.TokenType");

sas.stress.wkbench.CellEvaluationUtil = {
	getLatestValue:function(cellRef){
		var value="";
		if(cellRef){
			var data=cellRef.getFilterData()/*.getScenarioData()*/;
			var infoData=cellRef.getInfoData();
			var row=cellRef.row;
			var col=cellRef.column;
			var key="$"+infoData.getLongColumn(col)+"$"+infoData.getLongRow(row);
			var overlay=data.ncFormulas[key];
			var dataKey=infoData.getDatakeyFromRowColumn(key);
			var cellValue=sas.stress.wkbench.model.Project.getCurrentProject().getCellValueByDataKey(dataKey);
			if(!overlay){//pure value
				value=cellValue;
			}else if(overlay.error){
				value=overlay.txt;
			}/*
			DOES NOT WORK WELL WITH S1302379
			wonder why I had the !overlay.timestamp in the first place ... SRINI
			else if(!overlay.timestamp&&(overlay.source&&overlay.source!=="allocationOverride")){//not committed overlay and not becuase of "allocationRule"
				value=overlay.origtxt;
			}*/else{//committed overlay
				var showEquals;
				// S1431431.. if text startsWith ' and does not contain a '! it is TEXT do not parse anything
				if (cellValue && (typeof(cellValue) === "string" && ((cellValue.startsWith("='") || cellValue.startsWith("'")) && cellValue.indexOf("'!") === -1))) {
					showEquals = false;
					if (cellValue.startsWith("='")) {
						return cellValue.substr(1);
					}
					return cellValue;
				}else {
					showEquals = (overlay.expr.type !== "NAME" && overlay.expr.type !== "PREFIX") ||
								(overlay.expr.name && overlay.expr.name.indexOf("$") !== -1);	// S1431431
				}
				try {
					value = (showEquals ? "=" : "") + sas.stress.wkbench.parser.ExpressionUtil.print(overlay.expr, $.proxy(data.printShortContext, data));
					if (showEquals && !_.isNull(cellValue/*data.data[row][col]*/) &&
						(!_.isNumber(cellValue/*data.data[row][col]*/) && !_.isBoolean(cellValue/*data.data[row][col]*/)&& !_.isString(cellValue/*data.data[row][col]*/))) {	//fixes for S1301860.. ignore numeric, text and boolean values
						value = cellValue/*data.data[row][col]*/ + " " + value;
					}
				} catch (e) {
					console.error("Unable to evaluate expression " + sas.stress.wkbench.parser.ExpressionUtil.toString(overlay.expr), overlay);
					value = "#NAME? " + overlay.txt;
				}
			}
		}
		return value;
	},
	getLatestExpr:function(cellRef){
		var expr=null;
		if(cellRef){
			var data=cellRef.getFilterData()/*.getScenarioData()*/;
			var infoData=cellRef.getInfoData();
			var row=cellRef.row;
			var col=cellRef.column;
			var key="$"+infoData.getLongColumn(col)+"$"+infoData.getLongRow(row);
			var overlay=data.ncFormulas[key];
			if(overlay){//not committed overlay
				expr=overlay.expr;
			}else{
				overlay=data.cFormulas[key];
				if(overlay){//committed overlay
					expr=overlay.expr;
				}//else pure value, no expr
			}
		}
		return expr;
	},
	validateDependencies:function(references,scenario){
		var results=[];
		var getChildren=function(refId){
			var data=refId.getFilterData()/*.getScenarioData()*/;
			var key="$"+refId.colLong+"$"+refId.rowLong;
			var overlay=data.ncFormulas[key]||data.cFormulas[key];
			if(overlay){
				var expression=overlay.expr;
				return sas.stress.wkbench.CellEvaluationUtil.getCellRefs(expression,refId.getSheetFilters()/*getScenario()*/);
			} else {
				return [];
			}
		};
		_.each(references,function(refId){
			sas.stress.wkbench.CellEvaluationUtil._traverseGraph(getChildren,refId,results);
			results.push(refId);
		});
		return results;
	},
	_traverseGraph:function(getChildren,refId,results){
		_.each(getChildren(refId)||[],function(dep){
			if(results.indexOf(dep)===-1){
				results.push(dep);
				sas.stress.wkbench.CellEvaluationUtil._traverseGraph(getChildren,dep,results);
			}
		});
		return results;
	},
	getCellRefs:function(expression,scenario,errorobj){
		if(expression){
			var refArrays=[];
			var nameExps=sas.stress.wkbench.parser.ExpressionUtil.collect(expression,"NAME");
			var i,nameExp;
			for(i=0;i<nameExps.length;i++){
				nameExp=nameExps[i];
				if(sas.stress.wkbench.parser.ExpressionUtil._getType(nameExp.name)==="REFERENCE"){
					refArrays.push(sas.stress.wkbench.model.CellRef.getCellRefs(nameExp.name,scenario));
				}
			}
			var refs=[];
			_.each(refArrays,function(refArray){
				refs=_.union(refs,refArray);
			});
			var nullfound=_.find(refs,function(cellRef){
				return cellRef===null ;
			});
			if (nullfound!==null){
				return _.uniq(refs,false,function(cellRef){
					return cellRef.toIdString();
				});
			}else{
				if (errorobj!==undefined) {
					errorobj.error = true;
				}
				return [];
			}
			return _.uniq(refs,false,function(cellRef){
				return cellRef.toIdString();
			});
		}
		return [];//none found
	},
	getCellRefStrs:function(expression,scenario){
		if(expression){
			var refArrays=[];
			var nameExps=sas.stress.wkbench.parser.ExpressionUtil.collect(expression,"NAME");
			var i,nameExp;
			for(i=0;i<nameExps.length;i++){
				nameExp=nameExps[i];
				if(sas.stress.wkbench.parser.ExpressionUtil._getType(nameExp.name)==="REFERENCE"){
					try{
						refArrays.push(sas.stress.wkbench.model.CellRef.getCellRefStrs(nameExp.name,scenario));
					}catch(e){
						console.error(e);
					}
				}
			}
			var refs=[];
			_.each(refArrays,function(refArray){
				refs=_.union(refs,refArray);
			});
			return _.uniq(refs);//,false,function(cellRef){return cellRef.toIdString();});
		}
		return [];//none found
	},
	getFunctionReferences:function(expression){
		var refs=[];
		if(expression){
			var callExps=sas.stress.wkbench.parser.ExpressionUtil.collect(expression,"CALL");
			_.each(callExps,function(callExp){
				refs.push(callExp.func.name);
			});
		}
		return refs;
	},
	parseFormula:function(formulaStr){
		formulaStr=(formulaStr[0]==='=')?formulaStr.substring(1):formulaStr;
		var lexer=new sas.stress.wkbench.parser.Lexer(formulaStr);
		var parser=new sas.stress.wkbench.parser.Parser(lexer);
		return parser.parseExpression();
	},
	//Leaving this in for function name validation
	parseFunction:function(formula){
		if(formula===null||formula===undefined){
			return null;
		}
		if(formula.indexOf("=")===0){
			formula=formula.substring(1);
		}
		var lexer=new sas.stress.wkbench.parser.Lexer(formula);
		var parser=new sas.stress.wkbench.parser.Parser(lexer);
		var expression=parser.parseExpression();
		var funcRefs=sas.stress.wkbench.CellEvaluationUtil.getFunctionReferences(expression);//this changes things, now the function name can't be a dynamic expression
		var invalidNames=sas.stress.wkbench.CellEvaluationUtil.checkForInvalidFunctionNames(funcRefs);
		if(invalidNames.length){
			console.error("Invalid function name(s) found: "+funcRefs.join());
			throw "#NAME?";
		}
		return expression;
	},
	checkForInvalidFunctionNames:function(funcName){
		var funcNames=_.isArray(funcName)?funcName:[funcName];
		return _.filter(funcNames,function(name){
			return !sas.stress.wkbench.CellEvaluationUtil.functions[name.toLowerCase()];
		});
	},
	updateDependencies:function(scenario,thisCellRef,before,after, ignoreDependencies){
		var thisCellRefStr=thisCellRef.toIdString();
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var projectId=scenario.getSheetInfo().getSheet().getProject().getId();
		var project=sas.stress.wkbench.model.Project.getProject(projectId);
		var deps=project._overlayDeps;
		var refs=project._overlayRefs;
		var beforeRefs=[];
		try{
			beforeRefs=CellEvaluationUtil.getCellRefs(before||null,scenario);
		}catch(e){
			console.error("ignoring before refs problem!!");
		}
		var afterRefs=CellEvaluationUtil.getCellRefs(after,scenario);
		var refsToAdd=sas.stress.wkbench.StressUtil._differenceObj(afterRefs,beforeRefs);
		var refsToRemove=sas.stress.wkbench.StressUtil._differenceObj(beforeRefs,afterRefs);
		var i,cellRefStr;
		for(i=0;i<refsToAdd.length;i++){//update map cell->direct dependents. add itself as dependent
			cellRefStr=refsToAdd[i].toIdString();
			deps[cellRefStr]=_.union(deps[cellRefStr]||[],[thisCellRefStr]);
		}
		for(i=0;i<refsToRemove.length;i++){//update map cell->direct dependents. remove itself as dependent
			cellRefStr=refsToRemove[i].toIdString();
			deps[cellRefStr]=_.without(deps[cellRefStr]||[],thisCellRefStr);
		}
		refs[thisCellRefStr]=afterRefs;//update map cell->direct references to latest list of references
		var refsToRecalculate=sas.stress.wkbench.CellEvaluationUtil._collectReferences(deps,thisCellRefStr,{},ignoreDependencies);
		refsToRecalculate=sas.stress.wkbench.CellEvaluationUtil._removeRedundants(refs,refsToRecalculate);
		_.each(refsToRecalculate,function(cellRef){//reset all dependents
			var data=cellRef.getFilterData()/*.getScenarioData()*/;
			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var overlay=data.ncFormulas[key]||data.cFormulas[key];
			overlay.status=0;
		});
		_.each(refsToRecalculate,function(cellRef){//re-evaluate all dependents
			var data=cellRef.getFilterData()/*.getScenarioData()*/;
			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var overlay=data.ncFormulas[key]||data.cFormulas[key];
			data.evalOverlay(overlay);
		});
	},
	_removeRedundants:function(refs,refIdMap){
		var redundants={};
		_.each(refIdMap,function(refIdStr){
			_.each(refs[refIdStr]||[],function(referencedRefStr){
				redundants[referencedRefStr]=true;
			});
		});
		return _.filter(refIdMap,function(value,key){
			return !redundants[key];
		});
	},
	_collectReferences:function(deps,currentRefStr,set,ignoreDependencies){//collect all references
		var i;
		set=set||{};//Map to add unique cellRefs
		var dependents=deps[currentRefStr]||[];
		dependents=_.difference(dependents,ignoreDependencies);
		var dependentsLength=dependents.length;
		for(i=0;i<dependentsLength;i++){
			var cellRefStr=dependents[i];
			if(!set[cellRefStr]){
				set[cellRefStr]=sas.stress.wkbench.model.CellRef.parse(cellRefStr);
				sas.stress.wkbench.CellEvaluationUtil._collectReferences(deps,cellRefStr,set);
			}
		}
		return set;
	},
	saveNewFormula:function(formula,cellRef,save,ignoreDependencies){	// fixes for S1201112... added new parameter
		var CellEvaluationUtil=sas.stress.wkbench.CellEvaluationUtil;
		var newExpr=undefined;
		if(formula!==null){
			newExpr=CellEvaluationUtil.parseFormula(formula);
		} else{
			newExpr=CellEvaluationUtil.parseFormula("0.00");	// fummy value for not failing in the next lines
		}
		var newExprStr=JSON.stringify(newExpr);
		var scenario=cellRef.getSheetFilters()/*.getScenario()*/;
		var oldExpr=CellEvaluationUtil.getLatestExpr(cellRef);
		var oldExprStr=JSON.stringify(oldExpr);
		if(newExprStr!==oldExprStr||!save){	// fixes for S1203544... if new != old OR NOT save (log dialog discard)
			var errorObj={error:false};
			var error=false;
			try {
				var references = CellEvaluationUtil.getCellRefs(newExpr, scenario,errorObj);
				error=errorObj.error;
			}catch (e){
				error=true;
			}
			var dependencies=CellEvaluationUtil.validateDependencies(references,scenario);
			if(_.some(dependencies,cellRef.equals,cellRef)){
				throw "CYCLICAL_DEP";
			}
			var data=scenario.getData();
			var userName = sas.stress.wkbench.StressUtil.getUserName();
			var key="$"+cellRef.colLong+"$"+cellRef.rowLong;
			var txt="",showEquals=false;
			try {
				txt = (newExpr) ? sas.stress.wkbench.parser.ExpressionUtil.print(newExpr, $.proxy(data.printLongContext, data)) : null;
				if (txt){
					newExpr=CellEvaluationUtil.parseFormula(txt);	// store the long expr values for other sheets to evaluate
				}
				showEquals=(txt.startsWith("'") && txt.indexOf("'!") >=0) ? !txt.endsWith("'") : isNaN(txt);	// S1431431 && S1444361
			}catch(e){
				txt=formula;
			}
			var dataKey;
			if(data.ncFormulas.hasOwnProperty(key)){
				dataKey=data.ncFormulas[key].dataKey;
			}else{
				dataKey=scenario.getSheetInfo().getData().getDatakeyFromRowColumn(key);
			}
			var cellValue=sas.stress.wkbench.model.Project.getCurrentProject().getCellValueByDataKey(dataKey);
			var prevTxt=data.ncFormulas.hasOwnProperty(key)?data.ncFormulas[key].prevtxt:((data.cFormulas.hasOwnProperty(key)?data.cFormulas[key]:cellValue));
			var source="sheetOverride";
			var canBeDiscarded=true;
			if (ignoreDependencies){
				source="allocationOverride";	// rb.getText
				canBeDiscarded=false;
			}
			var overlay={
				user:userName,
				txt:(showEquals?"=":"")+txt,
				origtxt:formula,
				expr:newExpr,
				rowIndex:cellRef.row,
				columnIndex:cellRef.column,
				status:0,
				prevtxt:prevTxt,/*data.ncFormulas.hasOwnProperty(key)?data.ncFormulas[key].txt:data.data[cellRef.row][cellRef.column]*/
				error:error,
				dataKey:dataKey,
				source:source,
				canBeDiscarded:canBeDiscarded
			};
			if (save){	//S1201112.. do we need to create un-committed overlays?
				data.ncFormulas[key]=overlay;
			}
			data.evalOverlay(overlay);
			CellEvaluationUtil.updateDependencies(scenario, cellRef, oldExpr, newExpr, ignoreDependencies||[]);
		}
	},

	INVALID_ALLOCATION_VALUE:"invalid allocation value",
	functions:{
		"sum":function sum(){
			var values=Array.prototype.slice.call(arguments);
			var value=0;
			var sumItUp=function(num){
				if($.isArray(num)){
					_.each(num,sumItUp);
				}else if(_.isNumber(num)){
					value+=num;
				}
			};
			_.each(values,sumItUp);
			return value;
		},
		"if":function if_func(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length>1 && values.length<4){
				if(typeof values[0]==="boolean"){
					if(values[0]){
						return values[1];
					}else{
						return values.length>2?values[2]:false;
					}
				}else{
					return values.length>2&&values[0]===null?(values[2]===null?0:values[2]):(values[1]===null?0:values[1]);
				}
			}
			console.error("Error:Invalid syntax for If");
			throw "Error:Invalid syntax for If";
		},
		"abs":function abs(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length>0 && _.isNumber(values[0])/*isFinite(values[0])*/){
				return Math.abs(values[0]);
			}else{
				console.error("Error:Invalid syntax for Abs");
				throw "Error:Invalid syntax for Abs";
			}
		},
		"and":function and(){
			var values=Array.prototype.slice.call(arguments);
			var result=null;
			for(var i=0;i<values.length&&result!==false;i++){
				var val=values[i];
				if(_.isNumber(val)/*isFinite(val)*/){
					result=!!val;
				}else if(_.isBoolean(val)){
					result=val;
				}
			}
			if(result===null){
				console.error("Error:Invalid syntax for And");
				throw "Error:Invalid syntax for And";
			}
			return result;
		},
		"or":function or(){
			var values=Array.prototype.slice.call(arguments);
			var result=null;
			for(var i=0;i<values.length&&result!==true;i++){
				var val=values[i];
				if(_.isNumber(val)){
					result=!!val;
				}else if(_.isBoolean(val)){
					result=val;
				}
			}
			if(result===null){
				console.error("Error:Invalid syntax for And");
				throw "Error:Invalid syntax for And";
			}
			return result;
		},
		"average":function average(){
			var values=Array.prototype.slice.call(arguments);
			var value=0;
			var count=0;
			var nullcount=0;
			var sumItUp=function(num){
				if($.isArray(num)){
					_.each(num,sumItUp);
				}else if(_.isNull(num)){
					value+=num;
					nullcount++;
				}else if(_.isNumber(num)){
					value+=num;
					count++;
				}
			};
			_.each(values,sumItUp);
			if(count===0){
				if(nullcount===0) {
					console.error("Error:Invalid syntax for Average");
					throw "Error:Invalid syntax for Average";
				} else{
					console.error("Error:All values NULL for Average");
					throw {message:"#DIV/0!"};
				}
			}
			return value/count;
		},
		"var":function var_func(){
			var values=Array.prototype.slice.call(arguments);
			try{
				var mean=sas.stress.wkbench.CellEvaluationUtil.functions.average.apply(this,arguments);
			}catch(e){
				console.error("Error:Invalid syntax for Var");throw "Error:Invalid syntax for Var";
			}
			var meanSquared=mean*mean;
			var meanDouble=2*mean;
			var result=0;
			var count=0;
			var varItUp=function(val){
				if($.isArray(val)){
					_.each(val,varItUp);
				}else if(_.isNumber(val)){
					result+=val*(val-meanDouble)+meanSquared;
					count++;
				}
			};
			_.each(values,varItUp);
			return result/(count-1);
		},
		"stdev":function stdev(){
			var values=Array.prototype.slice.call(arguments);
			try{
				var variance=sas.stress.wkbench.CellEvaluationUtil.functions.var.apply(this,arguments);
			}catch(e){
				console.error("Error:Invalid syntax for Stdev");throw "Error:Invalid syntax for Stdev";
			}
			return Math.sqrt(variance);
		},
		"min":function min(){
			var values=Array.prototype.slice.call(arguments);
			var numbers=[];
			var pushIt=function(value){
				if($.isArray(value)){
					_.each(value,pushIt);
				}else if(_.isNumber(value)){
					numbers.push(value);
				}else if(_.isBoolean(value)){
					numbers.push(value?1:0);
				}
			};
			_.each(values,pushIt);
			if(numbers.length){
				return Math.min.apply(null,numbers);
			}else{
				console.error("Error:Invalid syntax for Min");
				throw "Error:Invalid syntax for Min";
			}
		},
		"max":function max(){
			var values=Array.prototype.slice.call(arguments);
			var numbers=[];
			var pushIt=function(value){
				if($.isArray(value)){
					_.each(value,pushIt);
				}else if(_.isNumber(value)){
					numbers.push(value);
				}else if(_.isBoolean(value)){
					numbers.push(value?1:0);
				}
			};
			_.each(values,pushIt);
			if(numbers.length){
				return Math.max.apply(null,numbers);
			}else{
				console.error("Error:Invalid syntax for Max");
				throw "Error:Invalid syntax for Max";
			}
		},
		"sumproduct":function sumproduct(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length<1 || !$.isArray(values[0])){
				console.error("Error:Invalid syntax for Sumproduct");
				throw "Error:Invalid syntax for Sumproduct";
			}
			var prev=values.shift();
			var length=prev.length;
			_.each(values,function(array){
				if(!$.isArray(array)||array.length!==length){
					console.error("Error:Invalid syntax for Sumproduct");
					throw "Error:Invalid syntax for Sumproduct";
				}
				for(var i=0;i<length;i++){
					if (_.isNumber(array[i])&&_.isNumber(prev[i])){	// make sure both are numbers S1270104 and other one
						prev[i] *= array[i];
					}else{
						prev[i]=0;	// ignore anything that is not number
					}
				}
			});
			var result=0;
			for(var i=0;i<length;i++){
				result+=prev[i];
			}
			return result;
		},
		"normdist":function normdist(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length<3 || !_.isNumber(values[0]) || !_.isNumber(values[1]) || !_.isNumber(values[2])){
				console.error("Error:Invalid syntax for Normdist");
				throw "Error:Invalid syntax for Normdist";
			}
			var value=values[0];
			var mean=values[1];
			var stdev=values[2];
			var variance=stdev*stdev;
			var cumulative=values.length>=4 && _.isBoolean(values[3]) && values[3];
			if(!cumulative){
				return 1/(Math.sqrt(2*Math.PI)*Math.sqrt(variance))*Math.exp(-1*(value-mean)*(value-mean)/(2*variance));
			}
			var x=(value-mean)/(Math.sqrt(2*variance));
			// save the sign of x
			var sign = (x >= 0) ? 1 : -1;
			x = Math.abs(x);
			// constants
			var a1 =  0.254829592;
			var a2 = -0.284496736;
			var a3 =  1.421413741;
			var a4 = -1.453152027;
			var a5 =  1.061405429;
			var p  =  0.3275911;
			// A&S formula 7.1.26
			var t = 1.0/(1.0 + p*x);
			var y = 1.0 - (((((a5 * t + a4) * t) + a3) * t + a2) * t + a1) * t * Math.exp(-x * x);
			var erf=sign * y; // erf(-x) = -erf(x);
			return 0.5*(1+erf);
		},
		"norminv":function norminv(){
			/// Original C++ implementation found at http://www.wilmott.com/messageview.cfm?catid=10&threadid=38771
			/// C# implementation found at http://weblogs.asp.net/esanchez/archive/2010/07/29/a-quick-and-dirty-implementation-of-excel-norminv-function-in-c.aspx
			/*
			 *     Compute the quantile function for the normal distribution.
			 *
			 *     For small to moderate probabilities, algorithm referenced
			 *     below is used to obtain an initial approximation which is
			 *     polished with a final Newton step.
			 *
			 *     For very large arguments, an algorithm of Wichura is used.
			 *
			 *  REFERENCE
			 *
			 *     Beasley, J. D. and S. G. Springer (1977).
			 *     Algorithm AS 111: The percentage points of the normal distribution,
			 *     Applied Statistics, 26, 118-121.
			 *
			 *      Wichura, M.J. (1988).
			 *      Algorithm AS 241: The Percentage Points of the Normal Distribution.
			 *      Applied Statistics, 37, 477-484.
			 */
			var values=Array.prototype.slice.call(arguments);
			if(values.length<3 || !_.isNumber(values[0]) || !_.isNumber(values[1]) || !_.isNumber(values[2]) || values[0]<=0 || values[0]>=1 || values[2]<=0){
				console.error("Error:Invalid syntax for Norminv");
				throw "Error:Invalid syntax for Norminv";
			}
			var probability=values[0];
			var mean=values[1];
			var stdev=values[2];
			if(stdev===0){
				return mean;
			}
			var r,val,q = probability - 0.5;
			/*-- use AS 241 --- */
			/* double ppnd16_(double *p, long *ifault)*/
			/*      ALGORITHM AS241  APPL. STATIST. (1988) VOL. 37, NO. 3

			 Produces the normal deviate Z corresponding to a given lower
			 tail area of P; Z is accurate to about 1 part in 10**16.
			 */
			if (Math.abs(q) <= .425){
				/* 0.075 <= p <= 0.925 */
				r = .180625 - q * q;
				val =
					q * (((((((r * 2509.0809287301226727 +
					33430.575583588128105) * r + 67265.770927008700853) * r +
					45921.953931549871457) * r + 13731.693765509461125) * r +
					1971.5909503065514427) * r + 133.14166789178437745) * r +
					3.387132872796366608)
					/ (((((((r * 5226.495278852854561 +
					28729.085735721942674) * r + 39307.89580009271061) * r +
					21213.794301586595867) * r + 5394.1960214247511077) * r +
					687.1870074920579083) * r + 42.313330701600911252) * r + 1);
			}else{
				/* closer than 0.075 from {0,1} boundary */

				/* r = min(p, 1-p) < 0.075 */
				if (q > 0)
					r = 1 - p;
				else
					r = p;

				r = Math.Sqrt(-Math.log(r));
				/* r = sqrt(-log(r))  <==>  min(p, 1-p) = exp( - r^2 ) */

				if (r <= 5){
					/* <==> min(p,1-p) >= exp(-25) ~= 1.3888e-11 */
					r += -1.6;
					val = (((((((r * 7.7454501427834140764e-4 +
						.0227238449892691845833) * r + .24178072517745061177) *
						r + 1.27045825245236838258) * r +
						3.64784832476320460504) * r + 5.7694972214606914055) *
						r + 4.6303378461565452959) * r +
						1.42343711074968357734)
						/ (((((((r *
						1.05075007164441684324e-9 + 5.475938084995344946e-4) *
						r + .0151986665636164571966) * r +
						.14810397642748007459) * r + .68976733498510000455) *
						r + 1.6763848301838038494) * r +
						2.05319162663775882187) * r + 1);
				}else{
					/* very close to  0 or 1 */
					r += -5;
					val = (((((((r * 2.01033439929228813265e-7 +
						2.71155556874348757815e-5) * r +
						.0012426609473880784386) * r + .026532189526576123093) *
						r + .29656057182850489123) * r +
						1.7848265399172913358) * r + 5.4637849111641143699) *
						r + 6.6579046435011037772)
						/ (((((((r *
						2.04426310338993978564e-15 + 1.4215117583164458887e-7) *
						r + 1.8463183175100546818e-5) * r +
						7.868691311456132591e-4) * r + .0148753612908506148525)
						* r + .13692988092273580531) * r +
						.59983220655588793769) * r + 1);
				}

				if (q < 0.0){
					val = -val;
				}
			}

			return mean + stdev * val;
		},
		"round":function round(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length>0 && (_.isNumber(values[0])&& _.isNumber(values[1]))){
				return sas.stress.wkbench.parser.Math.round(values[0], values[1]);
			}else{
				console.error("Error:Invalid syntax for ROUND");
				throw "Error:Invalid syntax for ROUND";
			}
		},
		"roundup":function roundUp(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length>0 && (_.isNumber(values[0])&& _.isNumber(values[1]))){
				return sas.stress.wkbench.parser.Math.roundUp(values[0], values[1]);
			}else{
				console.error("Error:Invalid syntax for ROUNDUP");
				throw "Error:Invalid syntax for ROUNDUP";
			}
		},
		"rounddown":function roundDown(){
			var values=Array.prototype.slice.call(arguments);
			if(values.length>0 && (_.isNumber(values[0])&& _.isNumber(values[1]))){
				return sas.stress.wkbench.parser.Math.roundDown(values[0], values[1]);
			}else{
				console.error("Error:Invalid syntax for ROUNDDOWN");
				throw "Error:Invalid syntax for ROUNDDOWN";
			}
		},
		"mround":function mround(){
			var values=Array.prototype.slice.call(arguments);
			var isGood;
			if(values.length>0 && (_.isNumber(values[0])&& _.isNumber(values[1]))){
				if (Number(values[0]>=0&&Number(values[1]>=0))){
					isGood=true;
				}else if (Number(values[0]<0&&Number(values[1]<0))){
					isGood=true;
				}else{
					isGood=false;
				}
				if (isGood) {
					return sas.stress.wkbench.parser.Math.roundTo(values[0], values[1]);
				}else{
					console.error("Error:Invalid syntax for MROUND");
					throw "Error:Invalid syntax for MROUND";
				}
			}else{
				console.error("Error:Invalid syntax for ROUNDTO");
				throw "Error:Invalid syntax for ROUNDTO";
			}
		},
		/*"iferror":function ifError(){
			var values=Array.prototype.slice.call(arguments);
			if (values.length<2){
				console.error("Error: Invalid syntax for IFERROR");
				throw ("Error: Invalid syntax for IFERROR");
			}
			var returnValue;
			switch(values[0]){
				case "#N/A":
				case "#VALUE!":
				case "#DIV/0!":
				case "#NUM!":
				case "#NAME?":
				case "#NULL!":
				case Infinity:
					returnValue= values[1];

				break;

				default:
					returnValue= values[0];

				break;
			}
			return returnValue;
		},*/
		"sumif": function sumIf(){
			var TokenType=sas.stress.wkbench.parser.TokenType;
			var values=Array.prototype.slice.call(arguments);
			var values=Array.prototype.slice.call(arguments);
			if (values.length===0||values.length>4){
				console.error("Error: Invalid syntax for SUMIF");
				throw ("Error: Invalid syntax for SUMIF");
			}
			var returnValue=0;
			var sumItUp=function(num){
				if($.isArray(num)){
					_.each(num,sumItUp);
				}else if(_.isNumber(num)){
					returnValue+=num;
				}
			};
			var filteredValues=[];
			var criteriaToken = arguments[2];
			var criteria = arguments[1];
			switch(criteriaToken){
				case TokenType.GREATER_THAN.punctuator:
					if (arguments[3]) {
						filteredValues= _.filter(arguments[3], function(value){
							return value > criteria;
						});
						_.each(filteredValues, sumItUp);
					}else{
						filteredValues= _.filter(arguments[0], function(value){
							return value > criteria;
						});
						_.each(filteredValues, sumItUp);
					}
					break;
				case TokenType.GREATER_THAN_EQUAL.punctuator:
					if (arguments[3]) {
						filteredValues= _.filter(arguments[3], function(value){
							return value >= criteria;
						});
						_.each(filteredValues, sumItUp);
					}else{
						filteredValues= _.filter(arguments[0], function(value){
							return value >= criteria;
						});
						_.each(filteredValues, sumItUp);
					}
					break;
				case TokenType.LESS_THAN.punctuator:
					if (arguments[3]) {
						filteredValues= _.filter(arguments[3], function(value){
							return value < criteria;
						});
						_.each(filteredValues, sumItUp);
					}else{
						filteredValues= _.filter(arguments[0], function(value){
							return value < criteria;
						});
						_.each(filteredValues, sumItUp);
					}
					break;
				case TokenType.LESS_THAN_EQUAL.punctuator:
					if (arguments[3]) {
						filteredValues= _.filter(arguments[3], function(value){
							return value <= criteria;
						});
						_.each(filteredValues, sumItUp);
					}else{
						filteredValues= _.filter(arguments[0], function(value){
							return value <= criteria;
						});
						_.each(filteredValues, sumItUp);
					}
					break;
				case TokenType.EQUAL.punctuator:
					if (arguments[3]) {
						filteredValues= _.filter(arguments[3], function(value){
							return value === criteria;
						});
						_.each(filteredValues, sumItUp);
					}else{
						filteredValues= _.filter(arguments[0], function(value){
							return value === criteria;
						});
						_.each(filteredValues, sumItUp);
					}
					break;
				case TokenType.NOT_EQUAL.punctuator:
					if (arguments[3]) {
						filteredValues= _.filter(arguments[3], function(value){
							return value !== criteria;
						});
						_.each(filteredValues, sumItUp);
					}else{
						filteredValues= _.filter(arguments[0], function(value){
							return value !== criteria;
						});
						_.each(filteredValues, sumItUp);
					}
					break;

				default:
					break;
			}

			return returnValue;
		},
		"countif": function countIf(){
			var TokenType=sas.stress.wkbench.parser.TokenType;
			var values=Array.prototype.slice.call(arguments);
			console.log("count if values: ", values);

			if (values.length===0||values.length>3){
				console.error("Error: Invalid syntax for COUNTIF");
				throw ("Error: Invalid syntax for COUNTIF");
			}
			var returnValue=0;

			var filteredValues=[];
			var criteriaToken = arguments[2];
			var criteria = arguments[1];
			switch(criteriaToken){
				case TokenType.GREATER_THAN.punctuator:
					filteredValues= _.filter(arguments[0], function(value){
						return value > criteria;
					});
					returnValue=filteredValues.length;
					break;
				case TokenType.GREATER_THAN_EQUAL.punctuator:
					filteredValues= _.filter(arguments[0], function(value){
						return value >= criteria;
					});
					returnValue=filteredValues.length;
					break;
				case TokenType.LESS_THAN.punctuator:
					filteredValues= _.filter(arguments[0], function(value){
						return value < criteria;
					});
					returnValue=filteredValues.length;
					break;
				case TokenType.LESS_THAN_EQUAL.punctuator:
					filteredValues= _.filter(arguments[0], function(value){
						return value <= criteria;
					});
					returnValue=filteredValues.length;
					break;
				case TokenType.EQUAL.punctuator:
					filteredValues= _.filter(arguments[0], function(value){
						return value === criteria;
					});
					returnValue=filteredValues.length;
					break;
				case TokenType.NOT_EQUAL.punctuator:
					filteredValues= _.filter(arguments[0], function(value){
						return value !== criteria;
					});
					returnValue=filteredValues.length;
					break;
				case TokenType.ASTERISK.punctuator:
					filteredValues= _.filter(arguments[0], function(value){
						return (value&&(value.length >0));
					});
					returnValue=filteredValues.length;
					break;


				default:
					break;
			}

			return returnValue;
		}/*,	RQSSTR-811? TYPE is not supported in POI version SAS supports as of RFW 3.2 (3.17)
		"type": function type(){
			var values=Array.prototype.slice.call(arguments);
			if (values.length===0||values.length>1){
				console.error("Error: Invalid syntax for TYPE");
				throw ("Error: Invalid syntax for TYPE");
			}

			switch (typeof(values[0])) {
				case "number":
					return 1;

				case "string":
					if (values[0]==="#NAME?" || (values[0].indexOf("#DIV")>=0) || values[0] === Number.POSITIVE_INFINITY || values[0] === Number.NEGATIVE_INFINITY){
						return 16;
					}else{
						return 2;
					}
			}
		}*/
	}
};