jQuery.sap.require("sas.stress.wkbench.StressUtil");

sas.stress.wkbench.StressAPI = {
    loadProjectTemplates: function () {
        var deferred = $.Deferred();
        $.getJSON("rest/projectTemplates", function (projectTemplates) {
            deferred.resolve(projectTemplates.items);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr, textStatus, error);
        });
        return deferred;
    },

    loadProjects: function () {
        var deferred = $.Deferred();
        $.getJSON("rest/projects", function (projects) {
            deferred.resolve(projects.items);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr, textStatus, error);
        });
        return deferred;
    },

    //loadProjectTemplate: function (projectTemplateId) {
    //    var deferred = $.Deferred();
    //    $.ajax(
    //        {
    //            url: "rest/projectTemplates/" + projectTemplateId,
    //            cache: false
    //        }
    //    ).done(function (data) {
    //        deferred.resolve(data);
    //    }).fail(function (jqXHR, textStatus, errorThrown) {
    //        jQuery.sap.log.error("loading project FAILURE: " + errorThrown);
    //        deferred.reject(jqXHR);
    //    });
    //    return deferred;
    //},

    loadProject: function (projectId) {
        var deferred = $.Deferred();
        $.ajax(
            {
                url: "rest/projects/" + projectId,
                cache: false
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading project FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    loadModels: function () {
        var deferred = $.Deferred();
        /*$.getJSON("rest/models", function(models) {
         deferred.resolve(models.items);
         })*/
        $.ajax(
            {
                type: 'GET',
                url: "rest/models/",
                dataType: "json",
                accepts: { json: "application/json;charset=utf-8"/*+" application/vnd.sas.collection+json"*/ }
            }
        ).done(function (models) {
            deferred.resolve(models.items);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr, textStatus, error);
        });
        return deferred;
    },

    loadSheetFilters: function (projectId, sheetId) {
        var url = "rest/projects/" + projectId + "/sheets/" + sheetId + "/filterValues";
        var deferred = $.Deferred();
        $.ajax(
            {
                type: 'GET',
                url: url,
                dataType: "json"
            }
        ).done(function (filters) {
            deferred.resolve(filters.items);
        }).fail(function (jqxhr, textStatus, error) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr, textStatus, error);
        });
        return deferred;
    },

    loadSheetsDataForProjectAndSelectedFilters: function (projectId, iterationNum, requestPayLoad,preferredSheetId) {   // S1347872.. the preferred sheetID
        var url;
        if (preferredSheetId) {
            url = "rest/projects/" + projectId + "/sheets/data/"+ preferredSheetId + "?iteration=" + iterationNum;
        }else{
            url="rest/projects/" + projectId + "/sheets" + "/data?iteration=" + iterationNum;
        }
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: url,
                data: JSON.stringify(requestPayLoad || []),
                dataType: "json"
            }
        ).done(function (allSheetsData) {
            deferred.resolve(allSheetsData);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadSheetForSelectedSlicers: function (projectId, sheetId, iterationNum, requestPayLoad) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: "rest/projects/" + projectId + "/sheets/" + sheetId + "/data?iteration=" + iterationNum,
                data: JSON.stringify(requestPayLoad),
                dataType: "json"
            }
        ).done(function (sheet) {
            deferred.resolve(sheet);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadSheet: function (projectId, sheetId) {
        var deferred = $.Deferred();
        $.ajax(
            {
                url: "rest/projects/" + projectId + "/sheets/" + sheetId,
                cache: false
            }
        ).done(function (sheet) {
            deferred.resolve(sheet);
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
         });
        return deferred;
    },

    loadData: function (projectId, sheetId, scenarioId, entityId, iterationNum) {
        var deferred = $.Deferred();
        var url = "rest/projects/" + projectId + "/sheets/" + sheetId + "/scenarios/" + scenarioId + "/data?entity=" + entityId + "&iteration=" + iterationNum;
        $.ajax(
            {
                url: url,
                cache: false
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
         });
        return deferred;
    },

    verifyProjectStatusWithServer: function (project) {
        var deferred = $.Deferred();
        var valid = false;
        $.ajax(
            {
                type: "HEAD",
                url: "rest/projects/" + project.getId(),
                cache: false
            }
        ).done(function (data, textStatus, jqXHR) {
            var eTag = Number(jqXHR.getResponseHeader("ETag").substring(1, (jqXHR.getResponseHeader("ETag").length - 1)));
            valid = (eTag === project.getLastModified());

            deferred.resolve(valid);
        }).fail(function (jqXHR) {
            deferred.reject(jqXHR);
        });

        return deferred;
    },

    exportToExcel: function (projectId, iterationNum, data) {
        var deferred = $.Deferred();
        var self = this;
        var exportURL = 'rest/reports/' + projectId + '/export?iteration=' + iterationNum;  // S1305906

        var xhr = new XMLHttpRequest();
        xhr.onreadystatechange=function(){
            switch(xhr.readyState){
                case 1: // OPENED
                    // set up request header and send the request
                    xhr.setRequestHeader("Content-type", "application/json;charset=utf-8");
                    xhr.send(JSON.stringify(data)); //
                    break;
                case 2:
                    switch(xhr.status){
                        case 200:
                            xhr.responseType = "blob";
                            break;
                        default:
                            xhr.responseType="json";
                            break;
                    }
                    break;
                case 4:
                    switch(xhr.status){
                        case 200:
                        {
                            var contentDispositionHeader = xhr.getResponseHeader("content-disposition");
                            var fileName = contentDispositionHeader.slice(contentDispositionHeader.indexOf("filename")).replace("filename=\"", "").replace("\"", "");
                            sas.stress.wkbench.StressUtil._downloadHandler(this.response, fileName);
                            deferred.resolve();
                        }
                            break;
                        default:
                        {
                            deferred.reject(xhr);
                        }
                            break;
                    }
                    break;
            }
        };
        xhr.open("POST", exportURL, true);

        return deferred;
    },

    exportLogToCsv: function (projectId, entityId, sheetId, scenarioId, rowId, columnId) {
        if (arguments.length === 2) {
            //// RequestMapping("/projects/{projectId}/history.csv");
            $("#xlsFrame").attr('src', 'rest/projects/' + projectId + '/history.csv' + '?entity=' + entityId);
        } else if (arguments.length === 3) {
            // RequestMapping("/projects/{projectId}/sheets/{sheetId}/history.csv")
            $("#xlsFrame").attr('src', 'rest/projects/' + projectId + '/sheets/' + sheetId + '/history.csv' + '?entity=' + entityId);
        } else if (arguments.length === 4) {
            // RequestMapping("/projects/{projectId}/sheets/{sheetId}/scenarios/{scenarioId}/history.csv")
            $("#xlsFrame").attr('src', 'rest/projects/' + projectId + '/sheets/' + sheetId + '/scenarios/' + scenarioId + '/history.csv' + '?entity=' + entityId);
        } else if (arguments.length === 6) {
            // RequestMapping("/projects/{projectId}/sheets/{sheetId}/scenarios/{scenarioId}/history.csv/?rowId='rowId'&columnId='columnId'")
            $("#xlsFrame").attr('src', 'rest/projects/' + projectId + '/sheets/' + sheetId + '/scenarios/' + scenarioId + '/history.csv' + '?rowId=' + rowId + '&columnId=' + columnId + '&entity=' + entityId);
        }
    },

    createProjectTemplate: function (projectTemplate) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: "rest/projectTemplates/",
                data: JSON.stringify(projectTemplate),
                dataType: "json"
            }
        ).done(function (newProjectTemplate) {
            deferred.resolve(newProjectTemplate);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    createProject: function (project) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: "rest/config/projects/",
                data: JSON.stringify(project),
                dataType: "json"
            }
        ).done(function (newProject) {
            deferred.resolve(newProject);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    editProjectTemplate: function (projectTemplate) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "PUT",
                url: "rest/projectTemplates/" + projectTemplate.id,
                data: JSON.stringify(projectTemplate),
                dataType: "json"
            }
        ).done(function (editedProjectTemplate) {
            deferred.resolve(editedProjectTemplate);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    editProject: function (project) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "PUT",
                url: "rest/config/projects/" + project.projectKey,
                data: JSON.stringify(project),
                dataType: "json"
            }
        ).done(function (editedProject) {
            deferred.resolve(editedProject);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteProjectTemplates: function (projectTemplateIdList) {
        var deferred = $.Deferred();
        if (projectTemplateIdList.length === 1){    // S1398605
            $.ajax(
                {
                    type: "DELETE",
                    url: "rest/projectTemplates/" + projectTemplateIdList[0]
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }else {
            $.ajax(
                {
                    headers: {
                        Accept: "application/json",
                        "Content-Type": "application/json"
                    },
                    type: "DELETE",
                    url: "rest/projectTemplates/",
                    data: JSON.stringify(projectTemplateIdList),
                    dataType: "json"
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }
        return deferred;
    },

    deleteProjects: function (projectIdList) {
        var deferred = $.Deferred();
        if (projectIdList.length === 1){    // S1398605
            $.ajax(
                {
                    type: "DELETE",
                    url: "rest/projects/" + projectIdList[0]
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }else {
            $.ajax(
                {
                    headers: {
                        Accept: "application/json",
                        "Content-Type": "application/json"
                    },
                    type: "DELETE",
                    url: "rest/config/projects/",
                    data: JSON.stringify(projectIdList),
                    dataType: "json"
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }
        return deferred;
    },

    deleteDimensionCrossingFile: function (projectId) {
        var deferred = $.Deferred();
        $.ajax(
            {
                type: "DELETE",
                url: "rest/projects/" + projectId + "/crossing",
            }
        ).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteDimensionCrossingFileFromProjectTemplate: function (projectId) {
        var deferred = $.Deferred();
        $.ajax(
            {
                type: "DELETE",
                url: "rest/projectTemplates/" + projectId + "/crossing",
            }
        ).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    getProjectAccessControls: function (projectId) {
        var deferred = $.Deferred();
        $.getJSON("rest/config/projects/" + projectId + "/accessControls", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, errorThrown) {
            jQuery.sap.log.error("loading access controls FAILURE: " + errorThrown);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    saveProjectAccessControls: function (projectId, workgroups) {
        var deferred = $.Deferred();
        var accessControlsJson = { workgroups: workgroups };
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "PUT",
                url: "rest/config/projects/" + projectId + "/accessControls",
                data: JSON.stringify(accessControlsJson),
                dataType: "json"
            }
        ).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    updateProjectLock: function(project, lockObj) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json"
                },
                type: "PUT",
                url: "rest/projects/" + project.getId() + "/lock",
                data: JSON.stringify(lockObj),
                dataType: "json"
            }
        ).done(function () {
                deferred.resolve();
        }).fail(function (jqXHR) {
                jQuery.sap.log.error("Lock project FAILURE: " + jqXHR.responseText);
        });
        return deferred;

    },

    // fixes for S1239109: no need to send the entity
    saveProject: function (project) {
        var deferred = $.Deferred();
        var uncommittedPayload = project.getUncommittedPayload();
        $.ajax(
            {
                type: "PUT",
                url: "rest/projects/" + project.getId(),	// fixes for S1239109: no need to send entity
                contentType: "application/json;charset=utf-8",
                headers: { "If-Match": project.getLastModified() }, /* this is a hack in the current jQuery version, better solution see setRequestHeader() */
                data: JSON.stringify(uncommittedPayload)

                /* This the latest and greatest way of doing it. Unfortunately, we are a little behind in versions.
                 * Whenever, we support the latest version, we should use this rather than the "headers" above..
                 * - Srini

                 * .beforeSend(function(jqXHR) {
                 //jqXHR.setRequestHeader("Accept", "application/json");
                 jqXHR.setRequestHeader('If-Match', last);
                 })*/
            }
        ).done(function (data) {
            jQuery.sap.log.info("Save project SUCCESS");
            project.setLastModified(data.updatedTimeMillis);
            project.commitChanges();//uses last modified date from project
            deferred.resolve();
        }).fail(function (jqXHR) {
            jQuery.sap.log.error("Save project FAILURE: " + jqXHR.responseText);
            deferred.reject(uncommittedPayload, jqXHR);
        });
        return deferred;
    },

    uploadProject: function (project, attachment) {
        var projectId = project.getId();
        var formData = new FormData();
        formData.append('file', attachment);
        formData.append('projectId', projectId);
        return $.ajax(
            {
                type: "POST",
                url: "rest/projects/" + projectId + "/upload",
                data: formData,
                headers: { "If-Match": project.getLastModified() },
                processData: false,
                contentType: false,
                dataType: "text",
                cache: false
            }
        );
    },

    createProjectIteration: function (project, taskId) {
        var formData = new FormData();
        if (taskId) {   // RQSSTR-331
            formData.append('taskId', taskId);
        }
        return $.ajax(
            {
                type: "POST",
                url: "rest/config/projects/" + project.getId() + "/nextiteration",
                data: formData,
                processData: false,
                contentType: false
            }
        );
    },

    loadSessionProperties: function () {
        var deferred = $.Deferred();
        $.getJSON("rest/config/session/", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
        });

        return deferred;
    },

    loadHistoricalTasks: function (processId) {
        var deferred = $.Deferred();
        $.getJSON("rest/tasks/processes/" + processId + "/log", function (data) {
            deferred.resolve(_.sortBy(data.items, "id"));
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
         });
        return deferred;
    },

    loadSummaryTasks: function (processId) {
        var deferred = $.Deferred();
        $.getJSON("rest/tasks/processes/" + processId + "/alltasks", function (data) {
            deferred.resolve(_.sortBy(data.items, "id"));
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
         });
        return deferred;
    },

    loadAllMyTasks: function (processId) {
        var deferred = $.Deferred();
        var taskURL = (processId && processId !== null) ? "rest/tasks/processes/" + processId + "/allmytasks" : "rest/tasks/allmytasks";

        $.getJSON(taskURL, function (data) {
            deferred.resolve(_.sortBy(data.items, "id"));
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
         });

        return deferred;
    },

    loadMyTasks: function (processId) {
        var deferred = $.Deferred();
        $.getJSON("rest/tasks/processes/" + processId + "/tasks", function (data) {
            deferred.resolve(data.items);
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
         });
        return deferred;
    },

    loadProcess: function (processId) {
        var deferred = $.Deferred();
        $.getJSON("rest/tasks/processes/" + processId, function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
         });
        return deferred;
    },

    loadSubDefinition: function (processId, nodeId) {
        var deferred = $.Deferred();
        $.getJSON("rest/tasks/definitions/" + processId + "/tasks/" + nodeId + "/subprocess", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadSubProcess: function (processId, nodeId) {
        var deferred = $.Deferred();
        $.getJSON("rest/tasks/processes/" + processId + "/tasks/" + nodeId + "/subprocess", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadProcessDefinitions: function () {
        var deferred = $.Deferred();
        $.getJSON("rest/tasks/definitions", function (data) {
            deferred.resolve(data.items);
        }).fail(function (jqxhr) {
         deferred.reject(jqxhr);
         });
        return deferred;
    },

    saveTaskStatus: function (taskAction, comment) {
        var task = taskAction.getTask();
        var processId = task.getProject().getProcessId();
        var deferred = $.Deferred();
        var payload = { comment: comment, statusKey: taskAction.getPayload() };
        $.ajax(
            {
                type: "PUT",
                url: "rest/tasks/processes/" + processId + "/tasks/" + task.getId(),
                contentType: "application/json;charset=utf-8",
                data: JSON.stringify(payload)
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    createTaskComment: function (task, comment) {
        var processId = task.getProject().getProcessId();
        var deferred = $.Deferred();
        var payload = { comment: comment };
        $.ajax(
            {
                type: "POST",
                url: "rest/tasks/processes/" + processId + "/tasks/" + task.getId() + "/comments",
                contentType: "application/json;charset=utf-8",
                data: JSON.stringify(payload)
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadTaskComments: function (task) {
        var deferred = $.Deferred();
        var taskId = task.getId();
        var processId = task.getProject().getProcessId();
        $.getJSON("rest/tasks/processes/" + processId + "/tasks/" + taskId + "/comments", function (comments) {
            deferred.resolve(comments.items);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },


    //tasks/processes/160002/tasks/0100/changeSummary
    loadTaskTransitionChangeSummary:function(taskAction){
        var deferred = $.Deferred();
        var task = taskAction.getTask();
        var processId = task.getProject().getProcessId();
        var taskId=task.getId();
        $.getJSON("rest/tasks/processes/" + processId + "/tasks/" + taskId + "/changeSummary", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, errorThrown) {
            jQuery.sap.log.error("loading access controls FAILURE: " + errorThrown);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteTaskComment: function (task, commentId) {
        var deferred = $.Deferred();
        var taskId = task.getId();
        var processId = task.getProject().getProcessId();
        $.ajax(
            {
                type: "DELETE",
                url: "rest/tasks/processes/" + processId + "/tasks/" + taskId + "/comments/" + commentId
            }
        ).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    editTaskComment: function (task, commentId, text) {
        var deferred = $.Deferred();
        var taskId = task.getId();
        var processId = task.getProject().getProcessId();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json"
                },
                type: "PUT",
                url: "rest/tasks/processes/" + processId + "/tasks/" + taskId + "/comments/",
                contentType: "application/json;charset=utf-8",
                data: JSON.stringify({ commentKey: commentId, comment: text }),
                dataType: "json"
            }
        ).done(function (json) {
            deferred.resolve(json);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    createTaskAttachment: function (task, attachment, name, description) {
        var project = task.getProject();
        var deferred = $.Deferred();
        var formData = new FormData();
        formData.append('file', attachment);
        formData.append('name', name);
        formData.append('description', description);
        $.ajax(
            {
                type: "POST",
                url: "rest/attachments/project/" + project.getId() + "/task/" + task.getId(),
                data: formData,
                processData: false,
                contentType: false
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    loadTaskAttachments: function (task) {
        var deferred = $.Deferred();
        var taskId = task.getId();
        var project = task.getProject();
        $.ajax({
                   url: "rest/attachments/project/" + project.getId() + "/task/" + taskId,
                   cache: false
               }).done(function (attachments) {
            deferred.resolve(attachments.items);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },
    runTask: function(task,scriptJSON,workGroup){
        var processId = task.getProject().getProcessId();
        var deferred = $.Deferred();
        var workGroupParm = (workGroup) ? "?workGroupName=" + encodeURI(workGroup) : "";
        $.ajax(
            {
                type: "POST",
                url: "rest/tasks/processes/" + processId + "/tasks/" + task.getId() + "/run" + workGroupParm,
                contentType: "application/json;charset=utf-8",
                data: JSON.stringify(scriptJSON)
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    downloadTaskAttachment: function (attachmentId) {
        // fixes for S1188266...
        return sas.stress.wkbench.StressUtil._downloadAttachment(attachmentId);
    },

    deleteTaskAttachment: function (attachmentId) {
        var deferred = $.Deferred();
        $.ajax(
            {
                type: "DELETE",
                url: "rest/attachments/" + attachmentId
            }
        ).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    editTaskAttachment: function (attachmentId, description) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json"
                },
                type: "PUT",
                url: "rest/attachments/" + attachmentId,
                contentType: "application/json;charset=utf-8",
                data: description,
                dataType: "json"
            }
        ).done(function (json) {
            deferred.resolve(json);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadSheetHistory: function (projectId, sheetId, requestPayLoad) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: "rest/projects/" + projectId + "/sheets/" + sheetId + "/history",
                data: JSON.stringify(requestPayLoad),
                dataType: "json"
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadScenarioHistory: function (projectId, sheetId, scenarioId, entityId) {
        var deferred = $.Deferred();
        $.ajax(
            {
                url: "rest/projects/" + projectId + "/sheets/" + sheetId + "/scenarios/" + scenarioId + "/history?entity=" + entityId,
                cache: false
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading sheet history FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    loadUserPreferences: function () {
        var deferred = $.Deferred();
        $.getJSON("rest/prefs", function (data) {
            deferred.resolve(data);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading user preferences FAILURE: " + errorThrown);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqXHR);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    patchUserPreferences: function (userPreferences) {
        var deferred = $.Deferred();
        $.ajax(
            {
                type: "PATCH",
                url: "rest/prefs",
                contentType: "application/json",
                data: JSON.stringify(userPreferences)
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("patching user preferences FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    runModels: function (projectKey, selectedList) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json"
                },
                type: "POST",
                url: "rest/models/" + projectKey + "/runSelected",
                contentType: "application/json;charset=utf-8",
                data: JSON.stringify(selectedList),
                dataType: "json"
            }
        ).done(function (keys) {
            deferred.resolve(keys.items);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    getModelResults: function (projectKey, resultsKey) {
        var deferred = $.Deferred();
        $.getJSON("rest/models/" + projectKey + "/status/" + resultsKey, function (data) {
            deferred.resolve(data);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading Model RESULTS FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    loadModelsByProject: function (projectkey) {
        var deferred = $.Deferred();
        $.getJSON("rest/models/" + projectkey, function (models) {
            deferred.resolve(models.items);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading MODELS FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    getModelLog: function (projectkey, modelkey) {
        var deferred = $.Deferred();
        $.ajax(
            {
                type: "GET", // //rest/{projectKey}/model/{modelKey}/log
                url: "rest/models/" + projectkey + "/model/" + modelkey + "/log", // "rest/models/"+modelkey+"/log",
                //contentType: "application/json;charset=UTF-8", do not send it will be urlencoded
                dataType: "text"
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading MODEL LOG FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });

        return deferred;
    },

    getModel: function (projectkey, modelkey) {
        var deferred = $.Deferred();
        /*$.getJSON("rest/models/"+projectkey+"/model/"+modelkey,function(data){
         deferred.resolve(data);
         })*/
        $.ajax(
            {
                type: 'GET',
                url: "rest/models/" + projectkey + "/model/" + modelkey,
                dataType: "json",
                accepts: { json: "application/json;charset=utf-8" }
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading MODEL FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });

        return deferred;
    },

    getModelCode: function (modelKey) {
        var deferred = $.Deferred();
        /*$.getJSON("rest/models/"+modelKey+"/code",function(data){
         deferred.resolve(data);
         })*/
        $.ajax(
            {
                type: 'GET',
                url: "rest/models/" + modelKey + "/code",
                dataType: "json",
                accepts: { json: "application/json;charset=utf-8" }
            }
        ).done(function (data) {
            deferred.resolve(data);
        }).fail(function (jqXHR, textstatus, error) {
            jQuery.sap.log.error("loading MODEL CODE FAILURE: " + error);
            deferred.reject(jqXHR);
        });

        return deferred;
    },

    /**
     * Loads the set of locations that models can be imported from.
     * @returns {deferred}  deferred object resolving the array of location names
     */
    loadModelImportLocations: function () {
        var deferred = $.Deferred();
        $.getJSON("rest/models/importLocations", function (data) {
            deferred.resolve(data.items);
        }).fail(function (jqXHR, textstatus, error) {
            jQuery.sap.log.error("GET Model IMPORT LOCATIONS FAILURE: " + error);
            deferred.reject(jqXHR);
        });

        return deferred;
    },

    importModels: function (modelImportLocationName) {
        var deferred = $.Deferred();
        var MODEL_CONTENT_TYPE = "application/vnd.sas.modinv.rest.model+json";
        $.ajax(
            {
                headers: {
                    "Accept": "application/json",
                    "Content-Type": MODEL_CONTENT_TYPE
                },
                type: "POST",
                url: "rest/models/load?baseDir=" + modelImportLocationName
            }
        ).done(function (result) {
            deferred.resolve(result);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });

        return deferred;
    },

    deleteModels: function (modelKeyList) {
        var deferred = $.Deferred();
        if (modelKeyList.length === 1){
            $.ajax(
                {
                    type: "DELETE",
                    url: "rest/models/" + modelKeyList[0]
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }else {
            $.ajax(
                {
                    headers: {
                        Accept: "application/json",
                        "Content-Type": "application/json"
                    },
                    type: "DELETE",
                    url: "rest/models/",
                    data: JSON.stringify(modelKeyList),
                    dataType: "json"
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }
        return deferred;
    },

    getModelAccessControls: function (modelId) {
        var deferred = $.Deferred();
        $.getJSON("rest/models/" + modelId + "/accessControls", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, errorThrown) {
            jQuery.sap.log.error("loading access controls FAILURE: " + errorThrown);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    loadModelGroups: function () {
        var deferred = $.Deferred();
        $.getJSON("rest/modelGroups", function (groupsJSON) {
            deferred.resolve(groupsJSON.items);
        }).fail(function (jqXHR, textStatus, errorThrown) {
            jQuery.sap.log.error("loading MODELGROUPS FAILURE: " + errorThrown);
            deferred.reject(jqXHR);
        });
        return deferred;
    },

    createModelGroup: function (requestPayLoad) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "POST",
                url: "rest/modelGroups",
                data: JSON.stringify(requestPayLoad),
                dataType: "json"
            }
        ).done(function (group) {
            deferred.resolve(group);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    editModelGroup: function (groupkey, requestPayLoad) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "PUT",
                url: "rest/modelGroups/" + groupkey,
                data: JSON.stringify(requestPayLoad),
                dataType: "json"
            }
        ).done(function (group) {
            deferred.resolve(group);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    getModelGroupAccessControls: function (modelGroupId) {
        var deferred = $.Deferred();
        $.getJSON("rest/modelGroups/" + modelGroupId + "/accessControls", function (data) {
            deferred.resolve(data);
        }).fail(function (jqxhr, textStatus, errorThrown) {
            jQuery.sap.log.error("loading access controls FAILURE: " + errorThrown);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    saveModelGroupAccessControls: function (modelGroupId, workgroups) {
        var deferred = $.Deferred();
        var accessControlsJson = { workgroups: workgroups };
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "PUT",
                url: "rest/modelGroups/" + modelGroupId + "/accessControls",
                data: JSON.stringify(accessControlsJson),
                dataType: "json"
            }
        ).done(function () {
            deferred.resolve();
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    },

    deleteModelGroups: function (groupIdList) {
        var deferred = $.Deferred();

        if (groupIdList.length === 1){  // S1398605
            $.ajax(
                {
                    type: "DELETE",
                    url: "rest/modelGroups/"+groupIdList[0]
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }else {
            $.ajax(
                {
                    headers: {
                        Accept: "application/json",
                        "Content-Type": "application/json;charset=utf-8"
                    },
                    type: "DELETE",
                    url: "rest/modelGroups",
                    data: JSON.stringify(groupIdList),
                    dataType: "json"
                }
            ).done(function () {
                deferred.resolve();
            }).fail(function (jqxhr) {
                jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
                deferred.reject(jqxhr);
            });
        }
        return deferred;
    },
    previewMGCode: function (json) {
        var deferred = $.Deferred();
        $.ajax(
            {
                headers: {
                    Accept: "application/json",
                    "Content-Type": "application/json;charset=utf-8"
                },
                type: "PUT",
                url: "rest/modelGroups/previewCode",
                data: JSON.stringify(json),
                dataType: "json"
            }
        ).done(function (preview) {
            deferred.resolve(preview);
        }).fail(function (jqxhr) {
            jQuery.sap.log.error("Request Failed. Response text: " + jqxhr.responseText);
            deferred.reject(jqxhr);
        });
        return deferred;
    }
};