/**
 *
 */
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sap.ui.core.format.DateFormat");
jQuery.sap.require("sas.stress.wkbench.ViewManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.hc.m.MessageToast");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.stress.wkbench.CellEvaluationUtil");
jQuery.sap.require("sas.stress.wkbench.parser.ExpressionUtil");

sap.ui.controller("sas.stress.wkbench.view.LogDialog", {
    onInit: function () {
        var view = this.getView();
        var that=this;

        new sas.risk.common.utils.LayoutService(this.getView().oTable, "SheetHistoryLogTable");

        var rb = sas.stress.wkbench.StressUtil.getBundle();
        var logDialogView = this.getView();
        var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var sheetId = wkbenchView.getSelectedSheetId();
        //var scenarioId = wkbenchView.getSelectedScenarioId();
        var sheet = project.getSheet(sheetId);
        var sheetName = sheet.getName();
        var liveModel = wkbenchView.getModel("live");
        var cellRef = liveModel.getProperty("/selected");
        var col = "", row = "";
        if (cellRef) {
            var slimsheet = liveModel.getProperty("/slimsheet");
            if (slimsheet) {
                col = cellRef.colShort;
                row = cellRef.rowShort;
            }
        } else {
            // cell is not selected, therefore, no need to enable the list
            view.oSelect.setEnabled(false);
        }
        var enabled = liveModel.getProperty("/toggleButtons") || false;
        logDialogView.setBusy(true);
        var nodatatxt = logDialogView.oTable.getNoData();
        logDialogView.oTable.setNoData(rb.getText("project.loading.data.message.txt"));
        var sheetInfo = sheet.getInfo();
        var sheetData = sheetInfo.getData();
        var data=sheetInfo.getFilters().getData();
        var expr;
        sheet.getSheetLogUpdates().done(function (updates) {
            //var uncommits = _.where(updates, { committed: false });
            _.each(updates, function (update) {
                //Display formatted values in the columns and unformatted values in the tooltip.
                if (update.cellValue) {
                    update.cellValue = update.cellValue.toString(); //Convert value to String else RichTooltip.setText fails.
                    expr=sas.stress.wkbench.CellEvaluationUtil.parseFormula(update.cellValue);
                    try {
                        update.cellShortValue=sas.stress.wkbench.parser.ExpressionUtil.print(expr, $.proxy(data.printShortContext, data));
                    } catch (e) {   // S1431482 if cannot evaluate return value from server
                        update.cellShortValue = "#NAME? " + update.cellValue;
                    }
                }
                if (update.previousCellValue) {
                    update.previousCellValue = update.previousCellValue.toString(); //Convert value to String else RichTooltip.setText fails.
                    expr=sas.stress.wkbench.CellEvaluationUtil.parseFormula(update.previousCellValue);
                    try {
                        update.cellPreviousShortValue=sas.stress.wkbench.parser.ExpressionUtil.print(expr, $.proxy(data.printShortContext, data));
                        if (update.cellPreviousShortValue===""){ // S1400932
                            update.cellPreviousShortValue=update.previousCellValue;
                        }
                    }catch(e) {   // S1431482 if cannot evaluate return value from server
                        update.cellPreviousShortValue="#NAME? " + update.previousCellValue;
                    }
                }
                var columnShortRef = "";
                try {
                    columnShortRef = sheetData.getShortColumn(sheetInfo.getColumnIndex(update.columnId));
                } catch (e) {
                }
                var rowShortRef = "";
                try {
                    rowShortRef = sheetData.getShortRow(sheetInfo.getRowIndex(update.rowId));
                } catch (e) {
                }

                update.cellReference = columnShortRef + rowShortRef;    // the cell short reference
                switch (update.scope) {
                    case "global":
                        update.textScope = rb.getText("historyDialog.scope.global.column.value.txt");
                        break;
                    case "project":
                        update.textScope = rb.getText("historyDialog.scope.project.column.value.txt");
                        break;
                }
                switch (update.source) {
                    case "calculatedField":
                        update.textSource = rb.getText("historyDialog.source.calculatedfield.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                    case "sheetOverride":
                        update.textSource = rb.getText("historyDialog.source.sheetoverride.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                    case "dataLoad":
                        update.textSource = rb.getText("historyDialog.source.dataload.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                    case "modelExecution":
                        update.textSource = rb.getText("historyDialog.source.modelExecution.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                    case "rollup":
                        update.textSource = rb.getText("historyDialog.source.rollup.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                    case "excelImport":
                        update.textSource = rb.getText("historyDialog.source.excelimport.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                    case "archiveImport":
                        update.textSource = rb.getText("historyDialog.source.archiveImport.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                    case "allocationOverride":
                        update.textSource=rb.getText("historyDialog.source.allocationOverride.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                    case "externalOverride":
                        update.textSource=rb.getText("historyDialog.source.externalOverride.column.value.txt");
                        if (update.propertyId){
                            update.textSource=update.textSource+" ("+update.propertyId+")";
                        }
                        break;
                }

                var formatter = new sas.stress.wkbench.model.Formatter(update.formatType, update.formatPrecision, update.formatScale);
                update.formattedFormula = (update.cellValue !== undefined) ? formatter.format(update.cellShortValue) : null;
                update.formattedPreviousFormula = (update.previousCellValue) ? formatter.format(update.cellPreviousShortValue) : null;  // fixes for S1315432

                if (!update.iterationNum&&!update.timestamp){   // give an iteration number for the un-committed values..
                    update.iterationNum=project.getCurrentIterationNo();
                }
            });
            var logModel = new sas.risk.common.models.RiskJSONModel({
                modelData: updates,
                sheet: sheet,
                sheetName: sheetName,
                selectedItems: [],
                column: col,
                row: row,
                enabled: !enabled
            });
            logModel.setSizeLimit(updates.length);
            logDialogView.setModel(logModel);
            if (col.length && row.length) {    // a cell was selected, let's default to that initially
                logModel.setProperty("/cellReference",[col+row]);

                var item = view.oSelect.getItemByKey("key3");
                view.oSelect.setSelectedItemId(item.getId());
                view.oSelect.fireChange({ selectedItem: item });//for some odd reason i had to manually fire it.
            }
            // done loading...
            logDialogView.setBusy(false);
            sas.stress.wkbench.ViewManager.getWorkbench().getController().closeBzDialog();
            logDialogView.oTable.setNoData(nodatatxt);
        }).fail(function (jqxhr) {
            sas.stress.wkbench.ViewManager.getWorkbench().getController().closeBzDialog();
            logDialogView.setBusy(false);
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
            //logDialogView.oTable.setNoData(rb.getText("project.loading.data.error.message.txt"));
            that.cancel();
        });
    },
    onExit: function () {
    },
    cancel: function (e) {
        var view = this.getView();
        view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "CANCEL" });
    },
    applyLogFilter: function (e) {
        var view = this.getView();
        var model = view.getModel();
        var table = view.oTable;
        var oSelect = view.oSelect;
        var binding, orFilters;

        var cellReferences=model.getProperty("/cellReference");
        switch (oSelect.getSelectedItem().getKey()) {
            case "key1":
            {  //Sheet level
                /*var sheetName = model.getProperty("/sheetName");

                 var oSheetColumn = sap.ui.getCore().byId("historyDialog.sheet");
                 table.filter(oSheetColumn, sheetName);*/
                orFilters = new sap.ui.model.Filter("cellReference", sap.ui.model.FilterOperator.NE, ""); // the filters on the visible columns OR'd
                binding = table.getBinding("rows");
                binding.filter(orFilters);
                if (!cellReferences){
                    model.setProperty("/userFilter",false);
                }else if(!cellReferences.length){
                    model.setProperty("/userFilter",false);
                }else{
                    model.setProperty("/userFilter",true);
                }
                model.setProperty("/cellReference",null);   // so that the listeners for the next line are fired
                var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
                var liveModel = wkbenchView.getModel("live");
                var cellRef = liveModel.getProperty("/selected");
                if (cellRef) {
                    model.setProperty("/cellReference",[cellRef.colShort+cellRef.rowShort]);
                }

                break;
            }
            case "key3": //cell level
            {
                //oCellreference = sap.ui.getCore().byId("historyDialog.cellReference");
                var cellRefFilters=[];
                _.each(cellReferences,function(cellReference){
                    cellRefFilters.push(new sap.ui.model.Filter("cellReference", sap.ui.model.FilterOperator.EQ, cellReference));
                });
                orFilters = new sap.ui.model.Filter(cellRefFilters); // the filters on the visible columns OR'd
                binding = table.getBinding("rows");
                binding.filter(orFilters);
                //table.filter(oCellreference, cellReference);
                model.setProperty("/userFilter",true);
                break;
            }
            default:
                break;
        }
    },
    logDialog_discardSelectedPressed: function (e) {
        var project = sas.stress.wkbench.model.Project.getCurrentProject();
        var dataKey, cellOverlay;
        var view = this.getView();
        var model = view.getModel();
        var selectedItems = model.getProperty("/selectedItems");	// contains the uncommitted history items
        var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
        var unCommittedDiscards=0;

        view.fireEvent(view.getId() + "_CLOSING", { "view": view, "action": "CANCEL" });
        // loop through
        _.each(selectedItems, function (selecteditem) {
            var cellRef = selecteditem.getHistoryCellRef();
            var data = cellRef.getSheetFilters().getData();//cellRef.getScenario().getData();
            var key = "$" + cellRef.colLong + "$" + cellRef.rowLong;
            /* begin fixes for S1201001 */
            var unCommittedType = selecteditem.getUnCommittedType();
            switch (unCommittedType) {
                case "formula":	// if it's a formula perform formula stuff
                {
                    var overlay = data.ncFormulas[key];
                    if (overlay) {
                        dataKey = overlay.dataKey;
                        var prevtxt = overlay.prevtxt;
                        // delete the non-committed overlay..
                        delete data.ncFormulas[key];
                        // the prevTxt is the new overlay.. this should be what it was before
                        //overlay=prevtxt;
                        // the status is 0.. we need to evaluate it
                        overlay.status = 0;

                        // the global needs to be updated
                        cellOverlay = project.getCellOverlayByDataKey(dataKey);
                        // we will need to re-evaluate, therefore, make the call to saveNewFormula
                        if (prevtxt) {  // S1348670
                            if (prevtxt.expr) {
                                sas.stress.wkbench.CellEvaluationUtil.saveNewFormula(prevtxt.txt, cellRef, false);	// fixes for S1201112
                                cellOverlay.value.parsedFormula = prevtxt.expr;
                                cellOverlay.value.value = prevtxt.txt;
                            } else {
                                sas.stress.wkbench.CellEvaluationUtil.saveNewFormula(prevtxt.toString(), cellRef, false);	// fixes for S1201112
                                cellOverlay.value.parsedFormula = undefined;
                                cellOverlay.value.value = prevtxt.toString();
                                project.setCellValueByDataKey(dataKey, +(prevtxt));
                            }
                            cellOverlay.value.timestamp=prevtxt.timestamp;  // S1348670.. restore the timestamp
                        } else {
                            sas.stress.wkbench.CellEvaluationUtil.saveNewFormula(null, cellRef, false);	// fixes for S1201112
                        }
                        // does the cell have comments that are mandatory
                        if (cellOverlay.comment&&cellOverlay.comment.isMandatory) {   // does cell have mandatory comment
                            // need to delete the comment from the global list of cell values based on dataKey
                            delete data.ncComments[key];
                            cellOverlay.comment = undefined;

                            // clear the unsaved comments for the cell reference
                            wkbenchView.getScenarioEditorView().getController().clearUnSavedComments(cellRef); // S1344598
                            unCommittedDiscards++;  // one extra because un-saved mandatory comment is being deleted!!
                        }
                        if (!cellOverlay.value.timestamp) {
                            if (!cellOverlay.comment) {
                                delete project._overlays[dataKey];
                            }
                            delete project._values[dataKey];
                        }
                    }
                    unCommittedDiscards++;
                }
                    break;

                case "comment":	//if comment delete the comment.. the value stays as is
                {
                    // S1344598
                    dataKey = cellRef.getInfoData().getDatakeyFromRowColumn(key);
                    cellOverlay = project.getCellOverlayByDataKey(dataKey);
                    /*&&(cellOverlay&&cellOverlay.comment&&true===cellOverlay.comment.isMandatory)*/
                    var isAMandatoryComment=cellOverlay&&cellOverlay.comment?cellOverlay.comment.isMandatory:false;
                    if (!isAMandatoryComment/*cellOverlay&&cellOverlay.comment&&false===cellOverlay.comment.isMandatory*/){
                        // if it is a mandatory comment and the value was deleted the comment was deleted and therefore, we are good
                        unCommittedDiscards++;
                    }
                    /*
                     * if a mandatory comment and the ncFormula does not have the formula.. the comment can be deleted
                     * else, if not a mandatory comment, it can be deleted
                     */
                    var commentCanBeDeleted=isAMandatoryComment?(!data.ncFormulas.hasOwnProperty(key)?true:false):true;
                    if (commentCanBeDeleted) {   // we should not delete the mandatory comments
                        // need to delete the comment from the global list of cell values based on dataKey
                        delete data.ncComments[key];
                        cellOverlay.comment = undefined;

                        // clear the unsaved comments for the cell reference
                        wkbenchView.getScenarioEditorView().getController().clearUnSavedComments(cellRef); // S1344598
                    }

                }
                    break;
            }
            /* end fixes for S1201001 */
        });
        var uncommits = _.where(model.getProperty("/modelData"), { committed: false });
        // if the selecteditems length is same as uncommittedcount.. save = "false" because user has deleted all uncommitted
        var bSave = !(unCommittedDiscards/*selectedItems.length*/ === uncommits.length);
        // update the workbench view..

        wkbenchView.getController().navChecker.setDirty(bSave);
        var liveModel = wkbenchView.getModel("live");
        liveModel.setProperty("/toggleButtons", bSave);
        liveModel.setProperty("/reloadDueDiscard", true);
        // need to refresh the sheets S1344598
        sas.stress.wkbench.ViewManager.getWorkbench().getController()._refreshSheets();

        var slimSheet = liveModel.getProperty("/slimsheet");
        slimSheet.rerender();
        // fire the cell selected for the last cell selected
        slimSheet.fireCellSelected({ // S1344598
            selected: true,
            row: slimSheet.getSlimsheetSource().selectedRow,
            col: slimSheet.getSlimsheetSource().selectedCol,
            filter: selectedItems[0].getHistoryCellRef().getSheetFilters()
        });

        var rb = sas.stress.wkbench.StressUtil.getBundle();
        // show the message toast
        sas.hc.m.MessageToast.show(rb.getText("log.dialog.discard.successful.msg.txt"), { at: sap.ui.core.Popup.Dock.CenterTop });
    },
    _showSelectedCellReference:function(column,row,cellReferences){
        var view=this.getView();
        var model=view.getModel();

        if (cellReferences&&cellReferences.length){
            var cellRefsAsString="";
            _.each(cellReferences,function(cellReference,i){
                if (i>0){
                    cellRefsAsString+=",";
                }
                cellRefsAsString+=cellReference;
            });
            return "("+cellRefsAsString+")";
        }
        return "";
    },
    _rowSelectionChange:function(e){
        var view=this.getView();
        var rowContext,historyitem,model,item ;
        var oSelect = view.oSelect;

        if (e.getSource().getSelectedIndices().length){
            model=e.getSource().getModel();
            var selectedArray=model.getProperty("/selectedItems")||[];
            rowContext=e.getParameter("rowContext");
            if (rowContext) {
                historyitem=rowContext.getObject();
                //.. fixes for S1288922
                var itemCommitted=historyitem.committed===undefined?true:historyitem.committed;
                var itemCanBeDiscarded=historyitem.canBeDiscarded===undefined?false:historyitem.canBeDiscarded;
                if (!itemCommitted&&itemCanBeDiscarded){
                    // save the row to be discarded.. do not add it twice
                    var itemIndex=-1;
                    var item=_.find(selectedArray, function(item,index){
                        itemIndex=index;
                        return (item===historyitem)?item:undefined;
                    });
                    if (!item) {
                        selectedArray.push(historyitem);
                    }else{
                        selectedArray.splice(itemIndex,1);
                        selectedArray.join();
                    }
                    model.setProperty("/selectedItems",selectedArray);	// the items that will be discarded
                }
            }
            // this will return the first item that is committed in the list of items selected
            //.. fixes for S1288922.. _.some returns true if found, else false
            var anyItemCommitted=_.some(e.getSource().getSelectedIndices(), function(idx){
                var committed=e.getSource().getContextByIndex(idx).getObject().committed;
                return committed===undefined?true:false;
            });

            // the button will be enabled if there are items in the list and no item is committed
            view.discardbutton.setEnabled((selectedArray.length>0) && !anyItemCommitted);	//fixes for S1288922
            if(historyitem) {
                var userCellRefs=model.getProperty("/cellReference")||[];
                // make sure no duplicates
                if (!(_.contains(userCellRefs,historyitem.cellReference,0))){
                    userCellRefs.push(historyitem.cellReference);
                    model.setProperty("/cellReference",null);   // so that the listeners for the next line are fired
                    model.setProperty("/cellReference",userCellRefs);

                    model.setProperty("/userFilter",true);
                }
            }
        }else{
            if (!model){
                model=view.getModel();
            }
            model.setProperty("/selectedItems",[]);
            model.setProperty("/cellReference",null);   // so that the listeners for the next line are fired
            var wkbenchView = sas.stress.wkbench.ViewManager.getWorkbench();
            var liveModel = wkbenchView.getModel("live");
            var cellRef = liveModel.getProperty("/selected");
            if (cellRef) {
                model.setProperty("/cellReference",[cellRef.colShort+cellRef.rowShort]);
                model.setProperty("/userFilter",true);
            }else{
                model.setProperty("/userFilter",false);
            }
        }
    },
    _enableSelectList:function(userFilter){
        return userFilter===undefined?false:userFilter;
    }
});