/*--------------------------------------------------------------------------------
   Copyright (C), 2016 SAS Institute Inc.  All Rights Reserved
   sst_import_project_data:  Import project data into the Risk and Finance Workbench

   inputs:  scheme     - http or https, defaults to http.  See https://en.wikipedia.org/wiki/Uniform_Resource_Locator 
                                        for the syntax (names of parts) of a URL
            hostname   - hostname of the server running the RFW, defaults to localhost
            port       - port number of the server running the RFW, defaults to 7980
            input_file - the server-tier file to upload containing the data. 
                         If your file is located in the public staging directory (that is, <SASConfig>/Lev1/AppData/SASRiskWorkGroup/groups/Public/SASRiskandFinanceWorkbench/staging)
                         input_file should be the following form:  public/<file name>
                         If your file is located in a work group's staging directory, 
                         input_file should be the following form:  <work group name>/<file name>
            project_id - the numeric ID of the project to load the data into (defaults to &sst_current_project_id - a macro variable available when running a model group or script).
            userid     - the userid to use for the HTTP call into the RFW mid-tier to perform the operation.
                         The userid/password combination is optional when this macro is executed within a SAS session created by
                         the mid-tier for executing an RFW Model Group or Script.
            password   - the password to use for the HTTP call into the RFW mid-tier to perform the operation.
                         The userid/password combination is optional when this macro is executed within a SAS session created by
                         the mid-tier for executing an RFW Model Group or Script.
            data_format- HierarchyBased, DimensionBased or Mapped based on how the data is supplied.  HierarchyBased is the oldest, is the default and is deprecated.  Please use DimensionBased or Mapped.
            data_load_map_name - name of the data load map to use to map the input data to li_key-based data.  This parameter is required when data_format is Mapped.
            data_source  - The optional source code to load the data with.  Value values are 
            	   D audit records show up as Data Load (default) 
            	   O audit records show up External Override
            outvar_http_status_code - an optional *global* macro variable to place the HTTP response's status code from the HTTP call to the RFW mid-tier 
            outvar_http_body - an optional *global* macro variable to place the HTTP response body from the HTTP call to the RFW mid-tier

   Notes:     - Do not quote any of the input parameters to this macro.
              - This macro is meant to be used with a workspace server running SAS Host (basic) authentication.
                One can supply the userid and password via parameters to this macro or, if run within a Script or Model Group,
                the userid and password will be picked up automatically from the mid-tier execution.
------------------------------------------------------------------------------------*/

options NOQUOTELENMAX;
%macro sst_import_project_data(scheme=http, hostname=localhost, port=7980, input_file=, project_id=&SST_CURRENT_PROJECT_ID, userid=, password=, outvar_http_status_code=, outvar_http_body=, data_format=HierarchyBased, data_load_map_name=, data_source=);

  %local url  request  source data_format_parm map_parm data_source_parm;
  
  %let data_format_parm=;
  %if "&data_format" ne "" %then %do;
    %let data_format_parm=%nrstr(&)dataFormat%nrstr(=)&data_format;
  %end;
  
  %let map_parm=;
  %if "&data_load_map_name" ne "" %then %do;
    %let data_load_map_name=%sysfunc(urlencode(&data_load_map_name));
    %let map_parm=%nrstr(&)dataLoadMapName%nrstr(=)%sysfunc(urlencode(%superq(data_load_map_name)));
  %end;
  
  %let data_source_parm=;
  %if "&data_source" ne "" %then %do;
      %let data_source_parm=%nrstr(&)dataPointSource%nrstr(=)&data_source;
  %end;
  
  %let url=%superq(scheme)://%superq(hostname):&port./SASRiskandFinanceWorkbench/rest/;
  %let request=projects/&project_id/projectDataFromServer?surrogateFilePath=%superq(input_file)%superq(data_format_parm)%superq(map_parm)%superq(data_source_parm);
  %let source=sst_import_project_data;
  
  proc lua restart infile="risk.stress.macro_api_entry"; run;

%mend;
