/*****************************************************************************************
 *
 * Copyright (C), 2018
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved. 
 *
 * NAME    : sst_lua_init.sas  
 * PURPOSE : This is a convenience macro to make using the lua apis easier. Call this macro
 *           before a proc lua call (without the restart argument) to have the following lua
 *           modules loaded in the session:
 *              rfw     = require 'risk.stress.rfw_api'
 *              rest    = require 'risk.stress.rest_service'
 *              utils   = require 'risk.common.rest_utils'
 *              json    = require 'risk.common.json'
 *              content = require 'risk.stress.rfw_content_utils'
 *              file    = require 'risk.common.file_utils' 
 *           Besides loading these modules, the function rfw.url_set() is also called.
 *
 * IMPORTANT: This does not affect the authentication - it will default to tokenauth and if
 *            you want some other form of authentication, you still need to call rest.set_auth()
 *
 * SUPPORT : stmajo         
 *
 * PARAMETERS :
 *
 *   infile     = Leave blank to use the lua_init module in the risk/stress folder. Otherwise, 
 *                pass the path of  lua module to use. If the file is in the lua search path you
 *                can pass the path relative to the lua search path location.
 *              
 * EXAMPLE:
      %sst_lua_init();
      proc lua; submit;
         -- Set authentication
         rest.set_auth('ticket','my_user_id','my_password')
         
         -- Get rfw projects
         local projects = rfw.project_get()
         if not projects then
            sas.print('%1zCould not get the projects')
            return
         end
         print('All projects (name only): ')
         for _,p in ipairs(projects.items) do
            print('     '..p.name)
         end
         
         --Make a custom rest call and parse the response; get name of each process defined in RFW
         local pass, process_str = rest.request("GET", "tasks/definitions")    
         if not pass then
            sas.print('%1zCould not get the list of processes')
         end
         local processes = json:decode(process_str).items or {}
         print('')
         print('RFW Processes:')
         for _,p in ipairs(processes) do
            print('     '..p.name)
         end  
      endsubmit; run;
 *****************************************************************************************/
%macro sst_lua_init(infile=risk/stress/lua_init);
   proc lua restart infile="&infile."; run;
%mend;
